from typing import Optional

from scalekit.core import CoreClient
from scalekit.v1.tools.tools_pb2 import *
from scalekit.v1.tools.tools_pb2_grpc import ToolServiceStub
from google.protobuf import empty_pb2


class ToolsClient:
    """Class definition for Tools Client"""

    def __init__(self, core_client: CoreClient):
        """
        Initializer for Tools Client

        :param core_client    : CoreClient Object
        :type                 : ``` obj ```
        :returns
            None
        """
        self.core_client = core_client
        self.tool_service = ToolServiceStub(
            self.core_client.grpc_secure_channel
        )



    def list_tools(
        self,
        filter: Optional[Filter] = None,
        page_size: Optional[int] = None,
        page_token: Optional[str] = None
    ) -> ListToolsResponse:
        """
        Method to list tools

        :param filter           : Filter parameters for listing tools
        :type                   : ``` Filter ```
        :param page_size        : Maximum number of tools to return per page
        :type                   : ``` int ```
        :param page_token       : Token from a previous response for pagination
        :type                   : ``` str ```

        :returns:
            List Tools Response
        """
        return self.core_client.grpc_exec(
            self.tool_service.ListTools.with_call,
            ListToolsRequest(
                filter=filter,
                page_size=page_size,
                page_token=page_token
            ),
        )





    def list_scoped_tools(
        self,
        identifier: str,
        filter: Optional[ScopedToolFilter] = None,
        page_size: Optional[int] = None,
        page_token: Optional[str] = None
    ) -> ListScopedToolsResponse:
        """
        Method to list scoped tools for a specific identifier

        :param identifier       : Identifier to scope the tools list
        :type                   : ``` str ```
        :param filter           : Filter parameters for scoped tools
        :type                   : ``` ScopedToolFilter ```
        :param page_size        : Maximum number of tools to return per page
        :type                   : ``` int ```
        :param page_token       : Token from a previous response for pagination
        :type                   : ``` str ```

        :returns:
            List Scoped Tools Response
        """
        return self.core_client.grpc_exec(
            self.tool_service.ListScopedTools.with_call,
            ListScopedToolsRequest(
                identifier=identifier,
                filter=filter,
                page_size=page_size,
                page_token=page_token
            ),
        )

    def execute_tool(
        self,
        tool_name: str,
        identifier: str,
        params: Optional[dict] = None,
        connected_account_id: Optional[str] = None
    ) -> ExecuteToolResponse:
        """
        Method to execute a tool using a connected account

        :param tool_name        : Name of the tool to execute
        :type                   : ``` str ```
        :param identifier       : Identifier of the connected account
        :type                   : ``` str ```
        :param params           : Parameters for tool execution
        :type                   : ``` dict ```
        :param connected_account_id : ID of the connected account to use for tool execution
        :type                   : ``` str ```

        :returns:
            Execute Tool Response
        """
        from google.protobuf import struct_pb2
        
        params_struct = None
        if params:
            params_struct = struct_pb2.Struct()
            params_struct.update(params)
        
        return self.core_client.grpc_exec(
            self.tool_service.ExecuteTool.with_call,
            ExecuteToolRequest(
                tool_name=tool_name,
                identifier=identifier,
                params=params_struct,
                connected_account_id=connected_account_id
            ),
        )


from typing import Optional, Dict, Any
from datetime import datetime
from pydantic import BaseModel, Field
from scalekit.v1.connected_accounts.connected_accounts_pb2 import ConnectedAccount as ProtoConnectedAccount, ConnectorStatus, ConnectorType


class ConnectedAccount(BaseModel):
    """Connected account information"""
    
    id: Optional[str] = Field(None, description="Unique connected account ID")
    identifier: Optional[str] = Field(None, description="Connected account identifier")
    provider: Optional[str] = Field(None, description="Provider name")
    status: Optional[str] = Field(None, description="Connection status")
    authorization_type: Optional[str] = Field(None, description="Authorization type")
    authorization_details: Optional[Dict[str, Any]] = Field(None, description="Authorization details")
    token_expires_at: Optional[datetime] = Field(None, description="Token expiry time")
    updated_at: Optional[datetime] = Field(None, description="Last updated time")
    connector: Optional[str] = Field(None, description="Connector name")
    last_used_at: Optional[datetime] = Field(None, description="Last used time")

    @classmethod
    def from_proto(cls, proto_account: ProtoConnectedAccount) -> 'ConnectedAccount':
        """
        Create ConnectedAccount from protobuf ConnectedAccount
        
        :param proto_account: The protobuf ConnectedAccount object
        :type proto_account: ProtoConnectedAccount
        
        :returns:
            ConnectedAccount instance
        """
        # Convert protobuf timestamps to datetime
        token_expires_at = None
        if proto_account.token_expires_at:
            token_expires_at = proto_account.token_expires_at.ToDatetime()
            
        updated_at = None
        if proto_account.updated_at:
            updated_at = proto_account.updated_at.ToDatetime()
            
        last_used_at = None
        if proto_account.last_used_at:
            last_used_at = proto_account.last_used_at.ToDatetime()

        # Convert authorization details
        authorization_details = None
        if proto_account.authorization_details:
            authorization_details = {}
            if proto_account.authorization_type == ConnectorType.OAUTH :
                oauth_token = proto_account.authorization_details.oauth_token
                authorization_details["oauth_token"] = {
                    "access_token": oauth_token.access_token,
                    "refresh_token": oauth_token.refresh_token,
                    "scopes": list(oauth_token.scopes)
                }
            else:
                static_auth = proto_account.authorization_details.static_auth
                # Convert protobuf Struct to dict
                from google.protobuf.json_format import MessageToDict
                authorization_details["static_auth"] = MessageToDict(static_auth.details)

        return cls(
            id=proto_account.id if proto_account.id else None,
            identifier=proto_account.identifier,
            provider=proto_account.provider,
            status= ConnectorStatus.Name(proto_account.status) if proto_account.status else None,
            authorization_type=ConnectorType.Name(proto_account.authorization_type) if proto_account.authorization_type else None,
            authorization_details=authorization_details,
            token_expires_at=token_expires_at,
            updated_at=updated_at,
            connector=proto_account.connector,
            last_used_at=last_used_at
        )


class GetConnectedAccountAuthResponse(BaseModel):
    """Get connected account auth response with one-to-one mapping to proto GetConnectedAccountByIdentifierResponse"""
    
    connected_account: Optional[ConnectedAccount] = Field(
        None,
        description="Connected account details"
    )

    @classmethod
    def from_proto(cls, proto_response) -> 'GetConnectedAccountAuthResponse':
        """
        Create GetConnectedAccountAuthResponse from protobuf GetConnectedAccountByIdentifierResponse
        
        :param proto_response: The protobuf GetConnectedAccountByIdentifierResponse object
        :type proto_response: GetConnectedAccountByIdentifierResponse (from connected_accounts_pb2)
        
        :returns:
            GetConnectedAccountAuthResponse instance
        """
        connected_account = None
        if proto_response.connected_account:
            connected_account = ConnectedAccount.from_proto(proto_response.connected_account)
            
        return cls(connected_account=connected_account)

    def to_dict(self) -> dict:
        """
        Convert to dictionary representation
        
        :returns:
            Dictionary representation of the response
        """
        return {
            "connected_account": self.connected_account.model_dump() if self.connected_account else None
        }

    class Config:
        """Pydantic configuration"""
        validate_assignment = True
        json_encoders = {
            datetime: lambda v: v.isoformat() if v else None
        }


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: scalekit/v1/options/options.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!scalekit/v1/options/options.proto\x12\x13scalekit.v1.options\x1a google/protobuf/descriptor.proto\"\x86\x01\n\nAuthOption\x12X\n\x13\x61uthentication_type\x18\x03 \x01(\x0e\x32\'.scalekit.v1.options.AuthenticationTypeR\x12\x61uthenticationType\x12\x1e\n\npermission\x18\x01 \x01(\tR\npermission*\xb4\x02\n\x12\x41uthenticationType\x12\x0b\n\x07\x42LOCKED\x10\x00\x12\x08\n\x04NONE\x10\x01\x12\r\n\tWORKSPACE\x10@\x12\x13\n\x0f\x43USTOMER_PORTAL\x10 \x12\x0b\n\x07SESSION\x10\x10\x12\x15\n\x11WORKSPACE_SESSION\x10P\x12%\n!WORKSPACE_SESSION_CUSTOMER_PORTAL\x10p\x12,\n(WORKSPACE_SESSION_CUSTOMER_PORTAL_CLIENT\x10t\x12\x08\n\x04USER\x10\x08\x12\n\n\x06\x43LIENT\x10\x04\x12\x12\n\x0eSESSION_CLIENT\x10\x14\x12\x1c\n\x18WORKSPACE_SESSION_CLIENT\x10T\x12\"\n\x1e\x43USTOMER_PORTAL_SESSION_CLIENT\x10\x34:b\n\x0b\x61uth_option\x12\x1e.google.protobuf.MethodOptions\x18\xd0\x86\x03 \x01(\x0b\x32\x1f.scalekit.v1.options.AuthOptionR\nauthOptionB6Z4github.com/scalekit-inc/scalekit/pkg/grpc/authoptionb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'scalekit.v1.options.options_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z4github.com/scalekit-inc/scalekit/pkg/grpc/authoption'
  _globals['_AUTHENTICATIONTYPE']._serialized_start=230
  _globals['_AUTHENTICATIONTYPE']._serialized_end=538
  _globals['_AUTHOPTION']._serialized_start=93
  _globals['_AUTHOPTION']._serialized_end=227
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: buf/validate/expression.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1d\x62uf/validate/expression.proto\x12\x0c\x62uf.validate\"V\n\nConstraint\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x18\n\x07message\x18\x02 \x01(\tR\x07message\x12\x1e\n\nexpression\x18\x03 \x01(\tR\nexpression\"E\n\nViolations\x12\x37\n\nviolations\x18\x01 \x03(\x0b\x32\x17.buf.validate.ViolationR\nviolations\"\x82\x01\n\tViolation\x12\x1d\n\nfield_path\x18\x01 \x01(\tR\tfieldPath\x12#\n\rconstraint_id\x18\x02 \x01(\tR\x0c\x63onstraintId\x12\x18\n\x07message\x18\x03 \x01(\tR\x07message\x12\x17\n\x07\x66or_key\x18\x04 \x01(\x08R\x06\x66orKeyBp\n\x12\x62uild.buf.validateB\x0f\x45xpressionProtoP\x01ZGbuf.build/gen/go/bufbuild/protovalidate/protocolbuffers/go/buf/validateb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'buf.validate.expression_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\022build.buf.validateB\017ExpressionProtoP\001ZGbuf.build/gen/go/bufbuild/protovalidate/protocolbuffers/go/buf/validate'
  _globals['_CONSTRAINT']._serialized_start=47
  _globals['_CONSTRAINT']._serialized_end=133
  _globals['_VIOLATIONS']._serialized_start=135
  _globals['_VIOLATIONS']._serialized_end=204
  _globals['_VIOLATION']._serialized_start=207
  _globals['_VIOLATION']._serialized_end=337
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: scalekit/v1/auth/auth.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from google.api import visibility_pb2 as google_dot_api_dot_visibility__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from protoc_gen_openapiv2.options import annotations_pb2 as protoc__gen__openapiv2_dot_options_dot_annotations__pb2
from scalekit.v1.commons import commons_pb2 as scalekit_dot_v1_dot_commons_dot_commons__pb2
from scalekit.v1.connections import connections_pb2 as scalekit_dot_v1_dot_connections_dot_connections__pb2
from scalekit.v1.options import options_pb2 as scalekit_dot_v1_dot_options_dot_options__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1bscalekit/v1/auth/auth.proto\x12\x10scalekit.v1.auth\x1a\x1b\x62uf/validate/validate.proto\x1a\x1cgoogle/api/annotations.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x1bgoogle/api/visibility.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a.protoc-gen-openapiv2/options/annotations.proto\x1a!scalekit/v1/commons/commons.proto\x1a)scalekit/v1/connections/connections.proto\x1a!scalekit/v1/options/options.proto\"0\n\x16ListAuthMethodsRequest\x12\x16\n\x06intent\x18\x01 \x01(\tR\x06intent\"Z\n\x17ListAuthMethodsResponse\x12?\n\x0c\x61uth_methods\x18\x01 \x03(\x0b\x32\x1c.scalekit.v1.auth.AuthMethodR\x0b\x61uthMethods\"\x94\x05\n\nAuthMethod\x12}\n\rconnection_id\x18\x01 \x01(\tBX\x92\x41L22Unique identifier for the connection to be toggledJ\x16\"conn_121312434123312\"\xbaH\x06r\x04\x10\x01\x18 R\x0c\x63onnectionId\x12P\n\x0f\x63onnection_type\x18\x02 \x01(\x0e\x32\'.scalekit.v1.connections.ConnectionTypeR\x0e\x63onnectionType\x12\x1a\n\x08provider\x18\x03 \x01(\tR\x08provider\x12\x93\x01\n\x13\x61uth_initiation_uri\x18\x04 \x01(\tBc\x92\x41V2\x1eURI to initiate the connectionJ4\"https://sso.acmecorp.com/sso/v1/oidc/conn_123/init\"\xbaH\x07r\x05\x10\x01\x18\xff\x01R\x11\x61uthInitiationUri\x12z\n\x11passwordless_type\x18\x05 \x01(\x0e\x32).scalekit.v1.connections.PasswordlessTypeB\x1d\x92\x41\x1a\x32\x11Passwordless typeJ\x05\"OTP\"H\x00R\x10passwordlessType\x88\x01\x01\x12W\n\x15\x63ode_challenge_length\x18\x06 \x01(\rB\x1e\x92\x41\x1b\x32\x16Length of the OTP codeJ\x01\x36H\x01R\x13\x63odeChallengeLength\x88\x01\x01\x42\x14\n\x12_passwordless_typeB\x18\n\x16_code_challenge_length\"m\n\x1a\x44iscoveryAuthMethodRequest\x12O\n\x11\x64iscovery_request\x18\x02 \x01(\x0b\x32\".scalekit.v1.auth.DiscoveryRequestR\x10\x64iscoveryRequest\"\xb3\x01\n\x10\x44iscoveryRequest\x12\x63\n\x05\x65mail\x18\x01 \x01(\tBM\x92\x41\x41\x32*user identifier like email or phone numberJ\x13\"john@acmecorp.com\"\xbaH\x06r\x04\x10\x03\x18\x64R\x05\x65mail\x12:\n\x06intent\x18\x02 \x01(\x0e\x32\x18.scalekit.v1.auth.IntentB\x08\xbaH\x05\x82\x01\x02\x10\x01R\x06intent\"\\\n\x1b\x44iscoveryAuthMethodResponse\x12=\n\x0b\x61uth_method\x18\x01 \x01(\x0b\x32\x1c.scalekit.v1.auth.AuthMethodR\nauthMethod\"\x1e\n\x1cGetAuthCustomizationsRequest\"o\n\x1dGetAuthCustomizationsResponse\x12N\n\x16\x63ustomization_settings\x18\x02 \x01(\x0b\x32\x17.google.protobuf.StructR\x15\x63ustomizationSettings\"N\n\x17GetAuthFeaturesResponse\x12\x33\n\x08\x66\x65\x61tures\x18\x01 \x01(\x0b\x32\x17.google.protobuf.StructR\x08\x66\x65\x61tures\"R\n\x19VerifyPasswordLessRequest\x12\x35\n\x07otp_req\x18\x02 \x01(\x0b\x32\x1c.scalekit.v1.auth.OTPRequestR\x06otpReq\"\x1c\n\x1aVerifyPasswordLessResponse\"i\n\nOTPRequest\x12[\n\x0e\x63ode_challenge\x18\x01 \x01(\tB4\x92\x41(2\x1cOTP sent to the user\'s emailJ\x08\"123456\"\xbaH\x06r\x04\x10\x05\x18\x06R\rcodeChallenge\"\xd4\x01\n\x1dListUserOrganizationsResponse\x12\x44\n\rorganizations\x18\x01 \x03(\x0b\x32\x1e.scalekit.v1.auth.OrganizationR\rorganizations\x12\x31\n\x04user\x18\x02 \x01(\x0b\x32\x1d.scalekit.v1.auth.UserDetailsR\x04user\x12:\n\x06intent\x18\x03 \x01(\x0e\x32\x18.scalekit.v1.auth.IntentB\x08\xbaH\x05\x82\x01\x02\x10\x01R\x06intent\"\x82\x04\n\x0cOrganization\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12+\n\x11membership_status\x18\x03 \x01(\tR\x10membershipStatus\x12\x37\n\x15invitation_invited_by\x18\x04 \x01(\tH\x00R\x13invitationInvitedBy\x88\x01\x01\x12U\n\x16invitation_accepted_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x01R\x14invitationAcceptedAt\x88\x01\x01\x12S\n\x15invitation_created_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x02R\x13invitationCreatedAt\x88\x01\x01\x12S\n\x15invitation_expires_at\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x03R\x13invitationExpiresAt\x88\x01\x01\x42\x18\n\x16_invitation_invited_byB\x19\n\x17_invitation_accepted_atB\x18\n\x16_invitation_created_atB\x18\n\x16_invitation_expires_at\"_\n\x0bUserDetails\x12\x14\n\x05\x65mail\x18\x01 \x01(\tR\x05\x65mail\x12\x1d\n\nfirst_name\x18\x02 \x01(\tR\tfirstName\x12\x1b\n\tlast_name\x18\x03 \x01(\tR\x08lastName\"\xc4\x01\n\x19SignupOrganizationRequest\x12+\n\x11organization_name\x18\x01 \x01(\tR\x10organizationName\x12\x1d\n\nfirst_name\x18\x02 \x01(\tR\tfirstName\x12\x1b\n\tlast_name\x18\x03 \x01(\tR\x08lastName\x12\x1b\n\tfull_name\x18\x04 \x01(\tR\x08\x66ullName\x12!\n\x0cphone_number\x18\x05 \x01(\tR\x0bphoneNumber\"r\n\x1aSignupOrganizationResponse\x12\'\n\x0forganization_id\x18\x01 \x01(\tR\x0eorganizationId\x12+\n\x11organization_name\x18\x02 \x01(\tR\x10organizationName\"\xd6\x02\n\x1dUpdateLoginUserDetailsRequest\x12v\n\rconnection_id\x18\x01 \x01(\tBQ\x92\x41\x45\x32+Connection ID. Unique ID for the connectionJ\x16\"conn_121312434123312\"\xbaH\x06r\x04\x10\x01\x18 R\x0c\x63onnectionId\x12\x90\x01\n\x10login_request_id\x18\x02 \x01(\tBf\x92\x41]2BLogin Request ID that was shared as part of authorization initiateJ\x17\"lri_73415099636808061\"\xbaH\x03\xc8\x01\x01R\x0eloginRequestId\x12*\n\x04user\x18\x03 \x01(\x0b\x32\x16.scalekit.v1.auth.UserR\x04user\"\x90\x0c\n\x04User\x12\x83\x01\n\x03sub\x18\x01 \x01(\tBq\x92\x41g2WSubject identifier for the user (typically a unique user ID from the identity provider)J\x0c\"1234567890\"\xbaH\x04r\x02\x10\x01R\x03sub\x12R\n\x05\x65mail\x18\x02 \x01(\tB<\x92\x41\x32\x32\x1cUser\'s primary email addressJ\x12\"user@example.com\"\xbaH\x04r\x02\x10\x01R\x05\x65mail\x12=\n\ngiven_name\x18\x03 \x01(\tB\x1e\x92\x41\x1b\x32\x11User\'s first nameJ\x06\"John\"R\tgivenName\x12=\n\x0b\x66\x61mily_name\x18\x04 \x01(\tB\x1c\x92\x41\x19\x32\x10User\'s last nameJ\x05\"Doe\"R\nfamilyName\x12\x88\x01\n\x0e\x65mail_verified\x18\x05 \x01(\x08\x42\x61\x92\x41^2VIndicates whether the user\'s email address has been verified by the identity provider.J\x04trueR\remailVerified\x12\x63\n\x0cphone_number\x18\x06 \x01(\tB@\x92\x41=2,User\'s primary phone number in E.164 format.J\r\"+1234567890\"R\x0bphoneNumber\x12\x95\x01\n\x15phone_number_verified\x18\x07 \x01(\x08\x42\x61\x92\x41^2UIndicates whether the user\'s phone number has been verified by the identity provider.J\x05\x66\x61lseR\x13phoneNumberVerified\x12\x42\n\x04name\x18\x08 \x01(\tB.\x92\x41+2\x1d\x46ull display name of the userJ\n\"John Doe\"R\x04name\x12\x62\n\x12preferred_username\x18\t \x01(\tB3\x92\x41\x30\x32#User\'s preferred username or handleJ\t\"johndoe\"R\x11preferredUsername\x12\x62\n\x07picture\x18\n \x01(\tBH\x92\x41\x45\x32!URL to the user\'s profile pictureJ \"https://example.com/avatar.jpg\"R\x07picture\x12X\n\x06gender\x18\x0b \x01(\tB@\x92\x41=23User\'s gender as reported by the identity provider.J\x06\"male\"R\x06gender\x12\x65\n\x06locale\x18\x0c \x01(\tBM\x92\x41J2?User\'s locale or language preference (IETF BCP 47 language tag)J\x07\"en-US\"R\x06locale\x12\x66\n\x06groups\x18\r \x03(\tBN\x92\x41K2/List of group names or IDs the user belongs to.J\x18[\"admins\", \"developers\"]R\x06groups\x12\xf2\x01\n\x11\x63ustom_attributes\x18\x0e \x01(\x0b\x32\x17.google.protobuf.StructB\xab\x01\x92\x41\xa7\x01\x32mCustom attributes for the user, represented as a key-value map. Used for additional identity provider claims.J6{\"department\": \"Engineering\", \"employee_id\": \"E12345\"}R\x10\x63ustomAttributes\"R\n\x14GetAuthStateResponse\x12:\n\nauth_state\x18\x01 \x01(\x0e\x32\x1b.scalekit.v1.auth.AuthStateR\tauthState*:\n\x06Intent\x12\x16\n\x12INTENT_UNSPECIFIED\x10\x00\x12\x0b\n\x07sign_in\x10\x01\x12\x0b\n\x07sign_up\x10\x02*\x9e\x03\n\tAuthState\x12\x1a\n\x16\x41UTH_STATE_UNSPECIFIED\x10\x00\x12\x1e\n\x1a\x41UTHENTICATION_IN_PROGRESS\x10\x01\x12\x19\n\x15ORGANIZATION_SWITCHER\x10\x02\x12\x19\n\x15ORGANIZATION_SELECTED\x10\x03\x12\x17\n\x13ORGANIZATION_SIGNUP\x10\x04\x12 \n\x1cORGANIZATION_SWITCHER_SIGNUP\x10\x05\x12\x1c\n\x18OTP_VERIFICATION_PENDING\x10\x06\x12\x13\n\x0fMAGIC_LINK_SENT\x10\x07\x12&\n\"LINK_SENT_OTP_VERIFICATION_PENDING\x10\x08\x12\x10\n\x0cOTP_VERIFIED\x10\t\x12\x11\n\rLINK_VERIFIED\x10\n\x12\x15\n\x11SSO_AUTHENTICATED\x10\x0b\x12\x14\n\x10ORG_USER_CREATED\x10\x0c\x12\x1c\n\x18\x41UTHENTICATION_COMPLETED\x10\r\x12\x19\n\x15\x41UTHENTICATION_FAILED\x10\x0e\x32\xba\x0e\n\x0b\x41uthService\x12\x89\x01\n\x0fListAuthMethods\x12(.scalekit.v1.auth.ListAuthMethodsRequest\x1a).scalekit.v1.auth.ListAuthMethodsResponse\"!\x82\xb5\x18\x02\x18\x01\x82\xd3\xe4\x93\x02\x15\x12\x13/api/v1/authmethods\x12\xab\x01\n\x13\x44iscoveryAuthMethod\x12,.scalekit.v1.auth.DiscoveryAuthMethodRequest\x1a-.scalekit.v1.auth.DiscoveryAuthMethodResponse\"7\x82\xb5\x18\x02\x18\x01\x82\xd3\xe4\x93\x02+\"\x16/api/v1/auth:discovery:\x11\x64iscovery_request\x12\xab\x01\n\x15VerifyPasswordLessOtp\x12+.scalekit.v1.auth.VerifyPasswordLessRequest\x1a,.scalekit.v1.auth.VerifyPasswordLessResponse\"7\x82\xb5\x18\x02\x18\x01\x82\xd3\xe4\x93\x02+\" /api/v1/auth/passwordless:verify:\x07otp_req\x12t\n\x12ResendPasswordless\x12\x16.google.protobuf.Empty\x1a\x16.google.protobuf.Empty\".\x82\xb5\x18\x02\x18\x01\x82\xd3\xe4\x93\x02\"\" /api/v1/auth/passwordless:resend\x12\x8a\x01\n\x15ListUserOrganizations\x12\x16.google.protobuf.Empty\x1a/.scalekit.v1.auth.ListUserOrganizationsResponse\"(\x82\xb5\x18\x02\x18\x01\x82\xd3\xe4\x93\x02\x1c\x12\x1a/api/v1/auth:organizations\x12\x95\x01\n\x12SignupOrganization\x12+.scalekit.v1.auth.SignupOrganizationRequest\x1a,.scalekit.v1.auth.SignupOrganizationResponse\"$\x82\xb5\x18\x02\x18\x01\x82\xd3\xe4\x93\x02\x18\"\x13/api/v1/auth:signup:\x01*\x12p\n\x0cGetAuthState\x12\x16.google.protobuf.Empty\x1a&.scalekit.v1.auth.GetAuthStateResponse\" \x82\xb5\x18\x02\x18\x01\x82\xd3\xe4\x93\x02\x14\x12\x12/api/v1/auth/state\x12[\n\x06Logout\x12\x16.google.protobuf.Empty\x1a\x16.google.protobuf.Empty\"!\x82\xb5\x18\x02\x18\x01\x82\xd3\xe4\x93\x02\x15\"\x13/api/v1/auth/logout\x12h\n\x10GetActiveSession\x12\x16.google.protobuf.Empty\x1a\x16.google.protobuf.Empty\"$\x82\xb5\x18\x02\x18\x01\x82\xd3\xe4\x93\x02\x18\x12\x16/api/v1/session:active\x12\xa3\x01\n\x15GetAuthCustomizations\x12..scalekit.v1.auth.GetAuthCustomizationsRequest\x1a/.scalekit.v1.auth.GetAuthCustomizationsResponse\")\x82\xb5\x18\x02\x18\x01\x82\xd3\xe4\x93\x02\x1d\x12\x1b/api/v1/auth:customizations\x12y\n\x0fGetAuthFeatures\x12\x16.google.protobuf.Empty\x1a).scalekit.v1.auth.GetAuthFeaturesResponse\"#\x82\xb5\x18\x02\x18\x01\x82\xd3\xe4\x93\x02\x17\x12\x15/api/v1/auth:features\x12\xcd\x02\n\x16UpdateLoginUserDetails\x12/.scalekit.v1.auth.UpdateLoginUserDetailsRequest\x1a\x16.google.protobuf.Empty\"\xe9\x01\x92\x41z\n\x0b\x43onnections\x12%Update User Details for login request\x1a%Update User Details for login requestJ\x1d\n\x03\x32\x30\x30\x12\x16\n\x14\x44\x65leted Successfully\x82\xb5\x18\x02\x18\x04\xfa\xd2\xe4\x93\x02\t\x12\x07PREVIEW\x82\xd3\xe4\x93\x02Q\"I/api/v1/connections/{connection_id}/auth-requests/{login_request_id}/user:\x04userB0Z.github.com/scalekit-inc/scalekit/pkg/grpc/authb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'scalekit.v1.auth.auth_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z.github.com/scalekit-inc/scalekit/pkg/grpc/auth'
  _globals['_AUTHMETHOD'].fields_by_name['connection_id']._loaded_options = None
  _globals['_AUTHMETHOD'].fields_by_name['connection_id']._serialized_options = b'\222AL22Unique identifier for the connection to be toggledJ\026\"conn_121312434123312\"\272H\006r\004\020\001\030 '
  _globals['_AUTHMETHOD'].fields_by_name['auth_initiation_uri']._loaded_options = None
  _globals['_AUTHMETHOD'].fields_by_name['auth_initiation_uri']._serialized_options = b'\222AV2\036URI to initiate the connectionJ4\"https://sso.acmecorp.com/sso/v1/oidc/conn_123/init\"\272H\007r\005\020\001\030\377\001'
  _globals['_AUTHMETHOD'].fields_by_name['passwordless_type']._loaded_options = None
  _globals['_AUTHMETHOD'].fields_by_name['passwordless_type']._serialized_options = b'\222A\0322\021Passwordless typeJ\005\"OTP\"'
  _globals['_AUTHMETHOD'].fields_by_name['code_challenge_length']._loaded_options = None
  _globals['_AUTHMETHOD'].fields_by_name['code_challenge_length']._serialized_options = b'\222A\0332\026Length of the OTP codeJ\0016'
  _globals['_DISCOVERYREQUEST'].fields_by_name['email']._loaded_options = None
  _globals['_DISCOVERYREQUEST'].fields_by_name['email']._serialized_options = b'\222AA2*user identifier like email or phone numberJ\023\"john@acmecorp.com\"\272H\006r\004\020\003\030d'
  _globals['_DISCOVERYREQUEST'].fields_by_name['intent']._loaded_options = None
  _globals['_DISCOVERYREQUEST'].fields_by_name['intent']._serialized_options = b'\272H\005\202\001\002\020\001'
  _globals['_OTPREQUEST'].fields_by_name['code_challenge']._loaded_options = None
  _globals['_OTPREQUEST'].fields_by_name['code_challenge']._serialized_options = b'\222A(2\034OTP sent to the user\'s emailJ\010\"123456\"\272H\006r\004\020\005\030\006'
  _globals['_LISTUSERORGANIZATIONSRESPONSE'].fields_by_name['intent']._loaded_options = None
  _globals['_LISTUSERORGANIZATIONSRESPONSE'].fields_by_name['intent']._serialized_options = b'\272H\005\202\001\002\020\001'
  _globals['_UPDATELOGINUSERDETAILSREQUEST'].fields_by_name['connection_id']._loaded_options = None
  _globals['_UPDATELOGINUSERDETAILSREQUEST'].fields_by_name['connection_id']._serialized_options = b'\222AE2+Connection ID. Unique ID for the connectionJ\026\"conn_121312434123312\"\272H\006r\004\020\001\030 '
  _globals['_UPDATELOGINUSERDETAILSREQUEST'].fields_by_name['login_request_id']._loaded_options = None
  _globals['_UPDATELOGINUSERDETAILSREQUEST'].fields_by_name['login_request_id']._serialized_options = b'\222A]2BLogin Request ID that was shared as part of authorization initiateJ\027\"lri_73415099636808061\"\272H\003\310\001\001'
  _globals['_USER'].fields_by_name['sub']._loaded_options = None
  _globals['_USER'].fields_by_name['sub']._serialized_options = b'\222Ag2WSubject identifier for the user (typically a unique user ID from the identity provider)J\014\"1234567890\"\272H\004r\002\020\001'
  _globals['_USER'].fields_by_name['email']._loaded_options = None
  _globals['_USER'].fields_by_name['email']._serialized_options = b'\222A22\034User\'s primary email addressJ\022\"user@example.com\"\272H\004r\002\020\001'
  _globals['_USER'].fields_by_name['given_name']._loaded_options = None
  _globals['_USER'].fields_by_name['given_name']._serialized_options = b'\222A\0332\021User\'s first nameJ\006\"John\"'
  _globals['_USER'].fields_by_name['family_name']._loaded_options = None
  _globals['_USER'].fields_by_name['family_name']._serialized_options = b'\222A\0312\020User\'s last nameJ\005\"Doe\"'
  _globals['_USER'].fields_by_name['email_verified']._loaded_options = None
  _globals['_USER'].fields_by_name['email_verified']._serialized_options = b'\222A^2VIndicates whether the user\'s email address has been verified by the identity provider.J\004true'
  _globals['_USER'].fields_by_name['phone_number']._loaded_options = None
  _globals['_USER'].fields_by_name['phone_number']._serialized_options = b'\222A=2,User\'s primary phone number in E.164 format.J\r\"+1234567890\"'
  _globals['_USER'].fields_by_name['phone_number_verified']._loaded_options = None
  _globals['_USER'].fields_by_name['phone_number_verified']._serialized_options = b'\222A^2UIndicates whether the user\'s phone number has been verified by the identity provider.J\005false'
  _globals['_USER'].fields_by_name['name']._loaded_options = None
  _globals['_USER'].fields_by_name['name']._serialized_options = b'\222A+2\035Full display name of the userJ\n\"John Doe\"'
  _globals['_USER'].fields_by_name['preferred_username']._loaded_options = None
  _globals['_USER'].fields_by_name['preferred_username']._serialized_options = b'\222A02#User\'s preferred username or handleJ\t\"johndoe\"'
  _globals['_USER'].fields_by_name['picture']._loaded_options = None
  _globals['_USER'].fields_by_name['picture']._serialized_options = b'\222AE2!URL to the user\'s profile pictureJ \"https://example.com/avatar.jpg\"'
  _globals['_USER'].fields_by_name['gender']._loaded_options = None
  _globals['_USER'].fields_by_name['gender']._serialized_options = b'\222A=23User\'s gender as reported by the identity provider.J\006\"male\"'
  _globals['_USER'].fields_by_name['locale']._loaded_options = None
  _globals['_USER'].fields_by_name['locale']._serialized_options = b'\222AJ2?User\'s locale or language preference (IETF BCP 47 language tag)J\007\"en-US\"'
  _globals['_USER'].fields_by_name['groups']._loaded_options = None
  _globals['_USER'].fields_by_name['groups']._serialized_options = b'\222AK2/List of group names or IDs the user belongs to.J\030[\"admins\", \"developers\"]'
  _globals['_USER'].fields_by_name['custom_attributes']._loaded_options = None
  _globals['_USER'].fields_by_name['custom_attributes']._serialized_options = b'\222A\247\0012mCustom attributes for the user, represented as a key-value map. Used for additional identity provider claims.J6{\"department\": \"Engineering\", \"employee_id\": \"E12345\"}'
  _globals['_AUTHSERVICE'].methods_by_name['ListAuthMethods']._loaded_options = None
  _globals['_AUTHSERVICE'].methods_by_name['ListAuthMethods']._serialized_options = b'\202\265\030\002\030\001\202\323\344\223\002\025\022\023/api/v1/authmethods'
  _globals['_AUTHSERVICE'].methods_by_name['DiscoveryAuthMethod']._loaded_options = None
  _globals['_AUTHSERVICE'].methods_by_name['DiscoveryAuthMethod']._serialized_options = b'\202\265\030\002\030\001\202\323\344\223\002+\"\026/api/v1/auth:discovery:\021discovery_request'
  _globals['_AUTHSERVICE'].methods_by_name['VerifyPasswordLessOtp']._loaded_options = None
  _globals['_AUTHSERVICE'].methods_by_name['VerifyPasswordLessOtp']._serialized_options = b'\202\265\030\002\030\001\202\323\344\223\002+\" /api/v1/auth/passwordless:verify:\007otp_req'
  _globals['_AUTHSERVICE'].methods_by_name['ResendPasswordless']._loaded_options = None
  _globals['_AUTHSERVICE'].methods_by_name['ResendPasswordless']._serialized_options = b'\202\265\030\002\030\001\202\323\344\223\002\"\" /api/v1/auth/passwordless:resend'
  _globals['_AUTHSERVICE'].methods_by_name['ListUserOrganizations']._loaded_options = None
  _globals['_AUTHSERVICE'].methods_by_name['ListUserOrganizations']._serialized_options = b'\202\265\030\002\030\001\202\323\344\223\002\034\022\032/api/v1/auth:organizations'
  _globals['_AUTHSERVICE'].methods_by_name['SignupOrganization']._loaded_options = None
  _globals['_AUTHSERVICE'].methods_by_name['SignupOrganization']._serialized_options = b'\202\265\030\002\030\001\202\323\344\223\002\030\"\023/api/v1/auth:signup:\001*'
  _globals['_AUTHSERVICE'].methods_by_name['GetAuthState']._loaded_options = None
  _globals['_AUTHSERVICE'].methods_by_name['GetAuthState']._serialized_options = b'\202\265\030\002\030\001\202\323\344\223\002\024\022\022/api/v1/auth/state'
  _globals['_AUTHSERVICE'].methods_by_name['Logout']._loaded_options = None
  _globals['_AUTHSERVICE'].methods_by_name['Logout']._serialized_options = b'\202\265\030\002\030\001\202\323\344\223\002\025\"\023/api/v1/auth/logout'
  _globals['_AUTHSERVICE'].methods_by_name['GetActiveSession']._loaded_options = None
  _globals['_AUTHSERVICE'].methods_by_name['GetActiveSession']._serialized_options = b'\202\265\030\002\030\001\202\323\344\223\002\030\022\026/api/v1/session:active'
  _globals['_AUTHSERVICE'].methods_by_name['GetAuthCustomizations']._loaded_options = None
  _globals['_AUTHSERVICE'].methods_by_name['GetAuthCustomizations']._serialized_options = b'\202\265\030\002\030\001\202\323\344\223\002\035\022\033/api/v1/auth:customizations'
  _globals['_AUTHSERVICE'].methods_by_name['GetAuthFeatures']._loaded_options = None
  _globals['_AUTHSERVICE'].methods_by_name['GetAuthFeatures']._serialized_options = b'\202\265\030\002\030\001\202\323\344\223\002\027\022\025/api/v1/auth:features'
  _globals['_AUTHSERVICE'].methods_by_name['UpdateLoginUserDetails']._loaded_options = None
  _globals['_AUTHSERVICE'].methods_by_name['UpdateLoginUserDetails']._serialized_options = b'\222Az\n\013Connections\022%Update User Details for login request\032%Update User Details for login requestJ\035\n\003200\022\026\n\024Deleted Successfully\202\265\030\002\030\004\372\322\344\223\002\t\022\007PREVIEW\202\323\344\223\002Q\"I/api/v1/connections/{connection_id}/auth-requests/{login_request_id}/user:\004user'
  _globals['_INTENT']._serialized_start=5189
  _globals['_INTENT']._serialized_end=5247
  _globals['_AUTHSTATE']._serialized_start=5250
  _globals['_AUTHSTATE']._serialized_end=5664
  _globals['_LISTAUTHMETHODSREQUEST']._serialized_start=423
  _globals['_LISTAUTHMETHODSREQUEST']._serialized_end=471
  _globals['_LISTAUTHMETHODSRESPONSE']._serialized_start=473
  _globals['_LISTAUTHMETHODSRESPONSE']._serialized_end=563
  _globals['_AUTHMETHOD']._serialized_start=566
  _globals['_AUTHMETHOD']._serialized_end=1226
  _globals['_DISCOVERYAUTHMETHODREQUEST']._serialized_start=1228
  _globals['_DISCOVERYAUTHMETHODREQUEST']._serialized_end=1337
  _globals['_DISCOVERYREQUEST']._serialized_start=1340
  _globals['_DISCOVERYREQUEST']._serialized_end=1519
  _globals['_DISCOVERYAUTHMETHODRESPONSE']._serialized_start=1521
  _globals['_DISCOVERYAUTHMETHODRESPONSE']._serialized_end=1613
  _globals['_GETAUTHCUSTOMIZATIONSREQUEST']._serialized_start=1615
  _globals['_GETAUTHCUSTOMIZATIONSREQUEST']._serialized_end=1645
  _globals['_GETAUTHCUSTOMIZATIONSRESPONSE']._serialized_start=1647
  _globals['_GETAUTHCUSTOMIZATIONSRESPONSE']._serialized_end=1758
  _globals['_GETAUTHFEATURESRESPONSE']._serialized_start=1760
  _globals['_GETAUTHFEATURESRESPONSE']._serialized_end=1838
  _globals['_VERIFYPASSWORDLESSREQUEST']._serialized_start=1840
  _globals['_VERIFYPASSWORDLESSREQUEST']._serialized_end=1922
  _globals['_VERIFYPASSWORDLESSRESPONSE']._serialized_start=1924
  _globals['_VERIFYPASSWORDLESSRESPONSE']._serialized_end=1952
  _globals['_OTPREQUEST']._serialized_start=1954
  _globals['_OTPREQUEST']._serialized_end=2059
  _globals['_LISTUSERORGANIZATIONSRESPONSE']._serialized_start=2062
  _globals['_LISTUSERORGANIZATIONSRESPONSE']._serialized_end=2274
  _globals['_ORGANIZATION']._serialized_start=2277
  _globals['_ORGANIZATION']._serialized_end=2791
  _globals['_USERDETAILS']._serialized_start=2793
  _globals['_USERDETAILS']._serialized_end=2888
  _globals['_SIGNUPORGANIZATIONREQUEST']._serialized_start=2891
  _globals['_SIGNUPORGANIZATIONREQUEST']._serialized_end=3087
  _globals['_SIGNUPORGANIZATIONRESPONSE']._serialized_start=3089
  _globals['_SIGNUPORGANIZATIONRESPONSE']._serialized_end=3203
  _globals['_UPDATELOGINUSERDETAILSREQUEST']._serialized_start=3206
  _globals['_UPDATELOGINUSERDETAILSREQUEST']._serialized_end=3548
  _globals['_USER']._serialized_start=3551
  _globals['_USER']._serialized_end=5103
  _globals['_GETAUTHSTATERESPONSE']._serialized_start=5105
  _globals['_GETAUTHSTATERESPONSE']._serialized_end=5187
  _globals['_AUTHSERVICE']._serialized_start=5667
  _globals['_AUTHSERVICE']._serialized_end=7517
# @@protoc_insertion_point(module_scope)



import os
import time
import unittest
from faker import Faker
from basetest import BaseTest
from scalekit.v1.auth.passwordless_pb2 import TemplateType, PasswordlessType


class TestPasswordless(BaseTest):
    """Test class for Passwordless authentication methods"""

    def setUp(self):
        """Set up test fixtures"""
        self.faker = Faker()
        self.test_email = "dhaneshbabu007@gmail.com"
        self.invalid_auth_request_id = "invalid-auth-request-id"
        self.invalid_code = "000000"
        self.invalid_link_token = "invalid-link-token"
        self.auth_request_id = None  # Will be set by send_passwordless_email tests

    def test_send_passwordless_email_with_string_template(self):
        """Test sending passwordless email with string template (Node.js-like API)"""
        try:
            response = self.scalekit_client.passwordless.send_passwordless_email(
                email=self.test_email,
                template="SIGNIN",  
                magiclink_auth_uri="https://myapp.com/auth/callback",
                state="integration-test-state",
                expires_in=1800,  # 30 minutes
                template_variables={
                    "appName": "Integration Test App",
                    "company": "Test Company",
                    "employeeID": "EMP123"
                }
            )

            # Assert response structure and content
            self.assertEqual(response[1].code().name, "OK")
            self.assertIsNotNone(response[0])
            self.assertIsNotNone(response[0].auth_request_id)
            self.assertTrue(len(response[0].auth_request_id) > 0)
            self.assertTrue(response[0].expires_at > 0)
            self.assertTrue(response[0].expires_in > 0)
            self.assertIsNotNone(response[0].passwordless_type)
            
            # Store auth_request_id for other tests
            self.auth_request_id = response[0].auth_request_id
            
            print(f"✓ Email sent successfully! Auth Request ID: {response[0].auth_request_id}")
            
        except Exception as e:
            self.skipTest(f"Skipping test due to error: {str(e)}")

    def test_send_passwordless_email_with_direct_enum(self):
        """Test sending passwordless email with direct enum (if users prefer explicit enums)"""
        try:
            response = self.scalekit_client.passwordless.send_passwordless_email(
                email=self.test_email,
                template=TemplateType.SIGNUP,  # Direct enum access
                magiclink_auth_uri="https://myapp.com/auth/callback",
                state="integration-test-state-enum",
                expires_in=1800,
                template_variables={
                    "appName": "Integration Test App",
                    "company": "Test Company"
                }
            )

            # Assert response structure and content
            self.assertEqual(response[1].code().name, "OK")
            self.assertIsNotNone(response[0])
            self.assertIsNotNone(response[0].auth_request_id)
            self.assertTrue(len(response[0].auth_request_id) > 0)
            self.assertTrue(response[0].expires_at > 0)
            self.assertTrue(response[0].expires_in > 0)
            self.assertIsNotNone(response[0].passwordless_type)
            
            print(f"✓ Email sent with direct enum! Auth Request ID: {response[0].auth_request_id}")
            
        except Exception as e:
            self.skipTest(f"Skipping test due to error: {str(e)}")



    def test_send_passwordless_email_invalid_template(self):
        """Test sending passwordless email with invalid template"""
        with self.assertRaises(ValueError) as context:
            self.scalekit_client.passwordless.send_passwordless_email(
                email=self.test_email,
                template="INVALID_TEMPLATE"
            )
        
        self.assertIn("Invalid template type", str(context.exception))
        print("✓ Invalid template validation works correctly")



    def test_resend_passwordless_email(self):
        """Test resending passwordless email"""
        try:
            # First send an email to get an auth request ID
            send_response = self.scalekit_client.passwordless.send_passwordless_email(
                email=self.test_email,
                template="SIGNIN",
                magiclink_auth_uri="https://myapp.com/auth/callback",
                state="integration-test-state",
                expires_in=1800,
                template_variables={
                    "appName": "Integration Test App",
                    "company": "Test Company"
                }
            )

            # Assert initial response
            self.assertEqual(send_response[1].code().name, "OK")
            self.assertIsNotNone(send_response[0])
            self.assertIsNotNone(send_response[0].auth_request_id)
            
            auth_request_id = send_response[0].auth_request_id

            # Try to verify with invalid code (should fail)
            try:
                verify_response = self.scalekit_client.passwordless.verify_passwordless_email(
                    code=self.invalid_code,
                    auth_request_id=auth_request_id
                )
                # If we get here, verification didn't fail as expected
                self.assertEqual(verify_response[1].code().name, "OK")
            except Exception:
                # Expected to fail with invalid code
                pass

            # Wait a bit before resending to avoid rate limiting
            time.sleep(2)

            # Resend the email
            resend_response = self.scalekit_client.passwordless.resend_passwordless_email(
                auth_request_id=auth_request_id
            )

            # Assert resend response
            self.assertEqual(resend_response[1].code().name, "OK")
            self.assertIsNotNone(resend_response[0])
            self.assertIsNotNone(resend_response[0].auth_request_id)
            self.assertTrue(resend_response[0].expires_at > 0)
            self.assertTrue(resend_response[0].expires_in > 0)
            
            print(f"✓ Email resent successfully! Auth Request ID: {resend_response[0].auth_request_id}")
            
        except Exception as e:
            self.skipTest(f"Skipping test due to error: {str(e)}")

    def test_verify_passwordless_email_invalid_code(self):
        """Test verifying passwordless email with invalid code"""
        try:
            response = self.scalekit_client.passwordless.verify_passwordless_email(
                code=self.invalid_code,
                auth_request_id=self.invalid_auth_request_id
            )
            
            # This should fail, but if it somehow passes, check the response
            if response[1].code().name == "OK":
                self.fail("Expected verification to fail with invalid code")
                
        except Exception as e:
            # Expected to fail with invalid code
            self.assertIsNotNone(e)
            print(f"✓ Invalid code verification correctly failed: {str(e)}")

    def test_verify_passwordless_email_invalid_link_token(self):
        """Test verifying passwordless email with invalid link token"""
        try:
            response = self.scalekit_client.passwordless.verify_passwordless_email(
                link_token=self.invalid_link_token
            )
            
            # This should fail, but if it somehow passes, check the response
            if response[1].code().name == "OK":
                self.fail("Expected verification to fail with invalid link token")
                
        except Exception as e:
            # Expected to fail with invalid link token
            self.assertIsNotNone(e)
            print(f"✓ Invalid link token verification correctly failed: {str(e)}")





    def test_resend_passwordless_email_invalid_auth_request_id(self):
        """Test resending passwordless email with invalid auth request ID"""
        try:
            response = self.scalekit_client.passwordless.resend_passwordless_email(
                auth_request_id=self.invalid_auth_request_id
            )
            
            # This should fail, but if it somehow passes, check the response
            if response[1].code().name == "OK":
                self.fail("Expected resend to fail with invalid auth request ID")
                
        except Exception as e:
            # Expected to fail with invalid auth request ID
            self.assertIsNotNone(e)
            print(f"✓ Invalid auth request ID resend correctly failed: {str(e)}")

    def test_passwordless_enums(self):
        """Test that passwordless enums are properly accessible"""
        # Test template type enums
        self.assertEqual(TemplateType.SIGNIN, 1)
        self.assertEqual(TemplateType.SIGNUP, 2)
        
        # Test passwordless type enums
        self.assertEqual(PasswordlessType.OTP, 1)
        self.assertEqual(PasswordlessType.LINK, 2)
        self.assertEqual(PasswordlessType.LINK_OTP, 3)
        
        print("✓ All passwordless enums are properly accessible")

    def test_template_type_conversion(self):
        """Test the template type conversion functionality"""
        from scalekit.passwordless import _convert_template_type
        
        # Test string conversions
        self.assertEqual(_convert_template_type("SIGNIN"), TemplateType.SIGNIN)
        self.assertEqual(_convert_template_type("SIGNUP"), TemplateType.SIGNUP)
        self.assertEqual(_convert_template_type("signin"), TemplateType.SIGNIN)  # Case insensitive
        self.assertEqual(_convert_template_type("SignUp"), TemplateType.SIGNUP)  # Case insensitive
        
        # Test enum passthrough
        self.assertEqual(_convert_template_type(TemplateType.SIGNIN), TemplateType.SIGNIN)
        
        # Test None handling
        self.assertIsNone(_convert_template_type(None))
        
        # Test invalid template
        with self.assertRaises(ValueError):
            _convert_template_type("INVALID")
            
        print("✓ Template type conversion works correctly")

    @classmethod
    def tearDownClass(cls):
        """Clean up after all tests"""
        print("\n✓ Passwordless tests completed")


if __name__ == '__main__':
    unittest.main() 


from buf.validate import validate_pb2 as _validate_pb2
from google.api import annotations_pb2 as _annotations_pb2
from google.api import field_behavior_pb2 as _field_behavior_pb2
from google.api import visibility_pb2 as _visibility_pb2
from google.protobuf import any_pb2 as _any_pb2
from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import empty_pb2 as _empty_pb2
from google.protobuf import struct_pb2 as _struct_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from protoc_gen_openapiv2.options import annotations_pb2 as _annotations_pb2_1
from scalekit.v1.commons import commons_pb2 as _commons_pb2
from scalekit.v1.options import options_pb2 as _options_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Role(_message.Message):
    __slots__ = ("id", "name", "display_name", "description", "default_creator", "default_member", "extends", "permissions", "dependent_roles_count", "is_org_role")
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    DISPLAY_NAME_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_CREATOR_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_MEMBER_FIELD_NUMBER: _ClassVar[int]
    EXTENDS_FIELD_NUMBER: _ClassVar[int]
    PERMISSIONS_FIELD_NUMBER: _ClassVar[int]
    DEPENDENT_ROLES_COUNT_FIELD_NUMBER: _ClassVar[int]
    IS_ORG_ROLE_FIELD_NUMBER: _ClassVar[int]
    id: str
    name: str
    display_name: str
    description: str
    default_creator: bool
    default_member: bool
    extends: str
    permissions: _containers.RepeatedCompositeFieldContainer[RolePermission]
    dependent_roles_count: int
    is_org_role: bool
    def __init__(self, id: _Optional[str] = ..., name: _Optional[str] = ..., display_name: _Optional[str] = ..., description: _Optional[str] = ..., default_creator: bool = ..., default_member: bool = ..., extends: _Optional[str] = ..., permissions: _Optional[_Iterable[_Union[RolePermission, _Mapping]]] = ..., dependent_roles_count: _Optional[int] = ..., is_org_role: bool = ...) -> None: ...

class CreateRole(_message.Message):
    __slots__ = ("name", "display_name", "description", "extends", "permissions")
    NAME_FIELD_NUMBER: _ClassVar[int]
    DISPLAY_NAME_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    EXTENDS_FIELD_NUMBER: _ClassVar[int]
    PERMISSIONS_FIELD_NUMBER: _ClassVar[int]
    name: str
    display_name: str
    description: str
    extends: str
    permissions: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, name: _Optional[str] = ..., display_name: _Optional[str] = ..., description: _Optional[str] = ..., extends: _Optional[str] = ..., permissions: _Optional[_Iterable[str]] = ...) -> None: ...

class CreateOrganizationRole(_message.Message):
    __slots__ = ("name", "display_name", "description", "extends", "permissions")
    NAME_FIELD_NUMBER: _ClassVar[int]
    DISPLAY_NAME_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    EXTENDS_FIELD_NUMBER: _ClassVar[int]
    PERMISSIONS_FIELD_NUMBER: _ClassVar[int]
    name: str
    display_name: str
    description: str
    extends: str
    permissions: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, name: _Optional[str] = ..., display_name: _Optional[str] = ..., description: _Optional[str] = ..., extends: _Optional[str] = ..., permissions: _Optional[_Iterable[str]] = ...) -> None: ...

class CreateRoleRequest(_message.Message):
    __slots__ = ("role",)
    ROLE_FIELD_NUMBER: _ClassVar[int]
    role: CreateRole
    def __init__(self, role: _Optional[_Union[CreateRole, _Mapping]] = ...) -> None: ...

class CreateRoleResponse(_message.Message):
    __slots__ = ("role",)
    ROLE_FIELD_NUMBER: _ClassVar[int]
    role: Role
    def __init__(self, role: _Optional[_Union[Role, _Mapping]] = ...) -> None: ...

class GetRoleRequest(_message.Message):
    __slots__ = ("role_name", "include")
    ROLE_NAME_FIELD_NUMBER: _ClassVar[int]
    INCLUDE_FIELD_NUMBER: _ClassVar[int]
    role_name: str
    include: str
    def __init__(self, role_name: _Optional[str] = ..., include: _Optional[str] = ...) -> None: ...

class GetRoleResponse(_message.Message):
    __slots__ = ("role",)
    ROLE_FIELD_NUMBER: _ClassVar[int]
    role: Role
    def __init__(self, role: _Optional[_Union[Role, _Mapping]] = ...) -> None: ...

class ListRolesRequest(_message.Message):
    __slots__ = ("include",)
    INCLUDE_FIELD_NUMBER: _ClassVar[int]
    include: str
    def __init__(self, include: _Optional[str] = ...) -> None: ...

class ListRolesResponse(_message.Message):
    __slots__ = ("roles",)
    ROLES_FIELD_NUMBER: _ClassVar[int]
    roles: _containers.RepeatedCompositeFieldContainer[Role]
    def __init__(self, roles: _Optional[_Iterable[_Union[Role, _Mapping]]] = ...) -> None: ...

class UpdateRole(_message.Message):
    __slots__ = ("display_name", "description", "extends", "permissions")
    DISPLAY_NAME_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    EXTENDS_FIELD_NUMBER: _ClassVar[int]
    PERMISSIONS_FIELD_NUMBER: _ClassVar[int]
    display_name: str
    description: str
    extends: str
    permissions: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, display_name: _Optional[str] = ..., description: _Optional[str] = ..., extends: _Optional[str] = ..., permissions: _Optional[_Iterable[str]] = ...) -> None: ...

class UpdateRoleRequest(_message.Message):
    __slots__ = ("role_name", "role")
    ROLE_NAME_FIELD_NUMBER: _ClassVar[int]
    ROLE_FIELD_NUMBER: _ClassVar[int]
    role_name: str
    role: UpdateRole
    def __init__(self, role_name: _Optional[str] = ..., role: _Optional[_Union[UpdateRole, _Mapping]] = ...) -> None: ...

class UpdateRoleResponse(_message.Message):
    __slots__ = ("role",)
    ROLE_FIELD_NUMBER: _ClassVar[int]
    role: Role
    def __init__(self, role: _Optional[_Union[Role, _Mapping]] = ...) -> None: ...

class DeleteRoleRequest(_message.Message):
    __slots__ = ("role_name", "reassign_role_id", "reassign_role_name")
    ROLE_NAME_FIELD_NUMBER: _ClassVar[int]
    REASSIGN_ROLE_ID_FIELD_NUMBER: _ClassVar[int]
    REASSIGN_ROLE_NAME_FIELD_NUMBER: _ClassVar[int]
    role_name: str
    reassign_role_id: str
    reassign_role_name: str
    def __init__(self, role_name: _Optional[str] = ..., reassign_role_id: _Optional[str] = ..., reassign_role_name: _Optional[str] = ...) -> None: ...

class CreateOrganizationRoleRequest(_message.Message):
    __slots__ = ("org_id", "role")
    ORG_ID_FIELD_NUMBER: _ClassVar[int]
    ROLE_FIELD_NUMBER: _ClassVar[int]
    org_id: str
    role: CreateOrganizationRole
    def __init__(self, org_id: _Optional[str] = ..., role: _Optional[_Union[CreateOrganizationRole, _Mapping]] = ...) -> None: ...

class CreateOrganizationRoleResponse(_message.Message):
    __slots__ = ("role",)
    ROLE_FIELD_NUMBER: _ClassVar[int]
    role: Role
    def __init__(self, role: _Optional[_Union[Role, _Mapping]] = ...) -> None: ...

class GetOrganizationRoleRequest(_message.Message):
    __slots__ = ("org_id", "role_name", "include")
    ORG_ID_FIELD_NUMBER: _ClassVar[int]
    ROLE_NAME_FIELD_NUMBER: _ClassVar[int]
    INCLUDE_FIELD_NUMBER: _ClassVar[int]
    org_id: str
    role_name: str
    include: str
    def __init__(self, org_id: _Optional[str] = ..., role_name: _Optional[str] = ..., include: _Optional[str] = ...) -> None: ...

class GetOrganizationRoleResponse(_message.Message):
    __slots__ = ("role",)
    ROLE_FIELD_NUMBER: _ClassVar[int]
    role: Role
    def __init__(self, role: _Optional[_Union[Role, _Mapping]] = ...) -> None: ...

class ListOrganizationRolesRequest(_message.Message):
    __slots__ = ("org_id", "include")
    ORG_ID_FIELD_NUMBER: _ClassVar[int]
    INCLUDE_FIELD_NUMBER: _ClassVar[int]
    org_id: str
    include: str
    def __init__(self, org_id: _Optional[str] = ..., include: _Optional[str] = ...) -> None: ...

class ListOrganizationRolesResponse(_message.Message):
    __slots__ = ("roles",)
    ROLES_FIELD_NUMBER: _ClassVar[int]
    roles: _containers.RepeatedCompositeFieldContainer[Role]
    def __init__(self, roles: _Optional[_Iterable[_Union[Role, _Mapping]]] = ...) -> None: ...

class UpdateOrganizationRoleRequest(_message.Message):
    __slots__ = ("org_id", "role_name", "role")
    ORG_ID_FIELD_NUMBER: _ClassVar[int]
    ROLE_NAME_FIELD_NUMBER: _ClassVar[int]
    ROLE_FIELD_NUMBER: _ClassVar[int]
    org_id: str
    role_name: str
    role: UpdateRole
    def __init__(self, org_id: _Optional[str] = ..., role_name: _Optional[str] = ..., role: _Optional[_Union[UpdateRole, _Mapping]] = ...) -> None: ...

class UpdateOrganizationRoleResponse(_message.Message):
    __slots__ = ("role",)
    ROLE_FIELD_NUMBER: _ClassVar[int]
    role: Role
    def __init__(self, role: _Optional[_Union[Role, _Mapping]] = ...) -> None: ...

class DeleteOrganizationRoleRequest(_message.Message):
    __slots__ = ("org_id", "role_name", "reassign_role_name")
    ORG_ID_FIELD_NUMBER: _ClassVar[int]
    ROLE_NAME_FIELD_NUMBER: _ClassVar[int]
    REASSIGN_ROLE_NAME_FIELD_NUMBER: _ClassVar[int]
    org_id: str
    role_name: str
    reassign_role_name: str
    def __init__(self, org_id: _Optional[str] = ..., role_name: _Optional[str] = ..., reassign_role_name: _Optional[str] = ...) -> None: ...

class GetRoleUsersCountRequest(_message.Message):
    __slots__ = ("role_name",)
    ROLE_NAME_FIELD_NUMBER: _ClassVar[int]
    role_name: str
    def __init__(self, role_name: _Optional[str] = ...) -> None: ...

class GetRoleUsersCountResponse(_message.Message):
    __slots__ = ("count",)
    COUNT_FIELD_NUMBER: _ClassVar[int]
    count: int
    def __init__(self, count: _Optional[int] = ...) -> None: ...

class GetOrganizationRoleUsersCountRequest(_message.Message):
    __slots__ = ("org_id", "role_name")
    ORG_ID_FIELD_NUMBER: _ClassVar[int]
    ROLE_NAME_FIELD_NUMBER: _ClassVar[int]
    org_id: str
    role_name: str
    def __init__(self, org_id: _Optional[str] = ..., role_name: _Optional[str] = ...) -> None: ...

class GetOrganizationRoleUsersCountResponse(_message.Message):
    __slots__ = ("count",)
    COUNT_FIELD_NUMBER: _ClassVar[int]
    count: int
    def __init__(self, count: _Optional[int] = ...) -> None: ...

class UpdateDefaultRolesRequest(_message.Message):
    __slots__ = ("default_creator", "default_member", "default_creator_role", "default_member_role")
    DEFAULT_CREATOR_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_MEMBER_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_CREATOR_ROLE_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_MEMBER_ROLE_FIELD_NUMBER: _ClassVar[int]
    default_creator: UpdateDefaultRole
    default_member: UpdateDefaultRole
    default_creator_role: str
    default_member_role: str
    def __init__(self, default_creator: _Optional[_Union[UpdateDefaultRole, _Mapping]] = ..., default_member: _Optional[_Union[UpdateDefaultRole, _Mapping]] = ..., default_creator_role: _Optional[str] = ..., default_member_role: _Optional[str] = ...) -> None: ...

class UpdateDefaultOrganizationRolesRequest(_message.Message):
    __slots__ = ("org_id", "default_member_role")
    ORG_ID_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_MEMBER_ROLE_FIELD_NUMBER: _ClassVar[int]
    org_id: str
    default_member_role: str
    def __init__(self, org_id: _Optional[str] = ..., default_member_role: _Optional[str] = ...) -> None: ...

class UpdateDefaultRolesResponse(_message.Message):
    __slots__ = ("default_creator", "default_member")
    DEFAULT_CREATOR_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_MEMBER_FIELD_NUMBER: _ClassVar[int]
    default_creator: Role
    default_member: Role
    def __init__(self, default_creator: _Optional[_Union[Role, _Mapping]] = ..., default_member: _Optional[_Union[Role, _Mapping]] = ...) -> None: ...

class UpdateDefaultOrganizationRolesResponse(_message.Message):
    __slots__ = ("default_member",)
    DEFAULT_MEMBER_FIELD_NUMBER: _ClassVar[int]
    default_member: Role
    def __init__(self, default_member: _Optional[_Union[Role, _Mapping]] = ...) -> None: ...

class UpdateDefaultRole(_message.Message):
    __slots__ = ("id", "name")
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    id: str
    name: str
    def __init__(self, id: _Optional[str] = ..., name: _Optional[str] = ...) -> None: ...

class Permission(_message.Message):
    __slots__ = ("id", "name", "description", "create_time", "update_time")
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    CREATE_TIME_FIELD_NUMBER: _ClassVar[int]
    UPDATE_TIME_FIELD_NUMBER: _ClassVar[int]
    id: str
    name: str
    description: str
    create_time: _timestamp_pb2.Timestamp
    update_time: _timestamp_pb2.Timestamp
    def __init__(self, id: _Optional[str] = ..., name: _Optional[str] = ..., description: _Optional[str] = ..., create_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., update_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class RolePermission(_message.Message):
    __slots__ = ("id", "name", "description", "create_time", "update_time", "role_name")
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    CREATE_TIME_FIELD_NUMBER: _ClassVar[int]
    UPDATE_TIME_FIELD_NUMBER: _ClassVar[int]
    ROLE_NAME_FIELD_NUMBER: _ClassVar[int]
    id: str
    name: str
    description: str
    create_time: _timestamp_pb2.Timestamp
    update_time: _timestamp_pb2.Timestamp
    role_name: str
    def __init__(self, id: _Optional[str] = ..., name: _Optional[str] = ..., description: _Optional[str] = ..., create_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., update_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., role_name: _Optional[str] = ...) -> None: ...

class CreatePermission(_message.Message):
    __slots__ = ("name", "description")
    NAME_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    name: str
    description: str
    def __init__(self, name: _Optional[str] = ..., description: _Optional[str] = ...) -> None: ...

class CreatePermissionRequest(_message.Message):
    __slots__ = ("permission",)
    PERMISSION_FIELD_NUMBER: _ClassVar[int]
    permission: CreatePermission
    def __init__(self, permission: _Optional[_Union[CreatePermission, _Mapping]] = ...) -> None: ...

class CreatePermissionResponse(_message.Message):
    __slots__ = ("permission",)
    PERMISSION_FIELD_NUMBER: _ClassVar[int]
    permission: Permission
    def __init__(self, permission: _Optional[_Union[Permission, _Mapping]] = ...) -> None: ...

class GetPermissionRequest(_message.Message):
    __slots__ = ("permission_name",)
    PERMISSION_NAME_FIELD_NUMBER: _ClassVar[int]
    permission_name: str
    def __init__(self, permission_name: _Optional[str] = ...) -> None: ...

class GetPermissionResponse(_message.Message):
    __slots__ = ("permission",)
    PERMISSION_FIELD_NUMBER: _ClassVar[int]
    permission: Permission
    def __init__(self, permission: _Optional[_Union[Permission, _Mapping]] = ...) -> None: ...

class UpdatePermissionRequest(_message.Message):
    __slots__ = ("permission_name", "permission")
    PERMISSION_NAME_FIELD_NUMBER: _ClassVar[int]
    PERMISSION_FIELD_NUMBER: _ClassVar[int]
    permission_name: str
    permission: CreatePermission
    def __init__(self, permission_name: _Optional[str] = ..., permission: _Optional[_Union[CreatePermission, _Mapping]] = ...) -> None: ...

class UpdatePermissionResponse(_message.Message):
    __slots__ = ("permission",)
    PERMISSION_FIELD_NUMBER: _ClassVar[int]
    permission: Permission
    def __init__(self, permission: _Optional[_Union[Permission, _Mapping]] = ...) -> None: ...

class ListPermissionsRequest(_message.Message):
    __slots__ = ("page_token", "page_size")
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    page_token: str
    page_size: int
    def __init__(self, page_token: _Optional[str] = ..., page_size: _Optional[int] = ...) -> None: ...

class ListPermissionsResponse(_message.Message):
    __slots__ = ("permissions", "prev_page_token", "next_page_token", "total_size")
    PERMISSIONS_FIELD_NUMBER: _ClassVar[int]
    PREV_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    TOTAL_SIZE_FIELD_NUMBER: _ClassVar[int]
    permissions: _containers.RepeatedCompositeFieldContainer[Permission]
    prev_page_token: str
    next_page_token: str
    total_size: int
    def __init__(self, permissions: _Optional[_Iterable[_Union[Permission, _Mapping]]] = ..., prev_page_token: _Optional[str] = ..., next_page_token: _Optional[str] = ..., total_size: _Optional[int] = ...) -> None: ...

class DeletePermissionRequest(_message.Message):
    __slots__ = ("permission_name",)
    PERMISSION_NAME_FIELD_NUMBER: _ClassVar[int]
    permission_name: str
    def __init__(self, permission_name: _Optional[str] = ...) -> None: ...

class ListRolePermissionsRequest(_message.Message):
    __slots__ = ("role_name",)
    ROLE_NAME_FIELD_NUMBER: _ClassVar[int]
    role_name: str
    def __init__(self, role_name: _Optional[str] = ...) -> None: ...

class ListRolePermissionsResponse(_message.Message):
    __slots__ = ("permissions",)
    PERMISSIONS_FIELD_NUMBER: _ClassVar[int]
    permissions: _containers.RepeatedCompositeFieldContainer[Permission]
    def __init__(self, permissions: _Optional[_Iterable[_Union[Permission, _Mapping]]] = ...) -> None: ...

class AddPermissionsToRoleRequest(_message.Message):
    __slots__ = ("role_name", "permission_names")
    ROLE_NAME_FIELD_NUMBER: _ClassVar[int]
    PERMISSION_NAMES_FIELD_NUMBER: _ClassVar[int]
    role_name: str
    permission_names: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, role_name: _Optional[str] = ..., permission_names: _Optional[_Iterable[str]] = ...) -> None: ...

class AddPermissionsToRoleResponse(_message.Message):
    __slots__ = ("permissions",)
    PERMISSIONS_FIELD_NUMBER: _ClassVar[int]
    permissions: _containers.RepeatedCompositeFieldContainer[Permission]
    def __init__(self, permissions: _Optional[_Iterable[_Union[Permission, _Mapping]]] = ...) -> None: ...

class RemovePermissionFromRoleRequest(_message.Message):
    __slots__ = ("role_name", "permission_name")
    ROLE_NAME_FIELD_NUMBER: _ClassVar[int]
    PERMISSION_NAME_FIELD_NUMBER: _ClassVar[int]
    role_name: str
    permission_name: str
    def __init__(self, role_name: _Optional[str] = ..., permission_name: _Optional[str] = ...) -> None: ...

class ListEffectiveRolePermissionsRequest(_message.Message):
    __slots__ = ("role_name",)
    ROLE_NAME_FIELD_NUMBER: _ClassVar[int]
    role_name: str
    def __init__(self, role_name: _Optional[str] = ...) -> None: ...

class ListEffectiveRolePermissionsResponse(_message.Message):
    __slots__ = ("permissions",)
    PERMISSIONS_FIELD_NUMBER: _ClassVar[int]
    permissions: _containers.RepeatedCompositeFieldContainer[Permission]
    def __init__(self, permissions: _Optional[_Iterable[_Union[Permission, _Mapping]]] = ...) -> None: ...

class ListDependentRolesRequest(_message.Message):
    __slots__ = ("role_name",)
    ROLE_NAME_FIELD_NUMBER: _ClassVar[int]
    role_name: str
    def __init__(self, role_name: _Optional[str] = ...) -> None: ...

class ListDependentRolesResponse(_message.Message):
    __slots__ = ("roles",)
    ROLES_FIELD_NUMBER: _ClassVar[int]
    roles: _containers.RepeatedCompositeFieldContainer[Role]
    def __init__(self, roles: _Optional[_Iterable[_Union[Role, _Mapping]]] = ...) -> None: ...

class DeleteRoleBaseRequest(_message.Message):
    __slots__ = ("role_name",)
    ROLE_NAME_FIELD_NUMBER: _ClassVar[int]
    role_name: str
    def __init__(self, role_name: _Optional[str] = ...) -> None: ...

class DeleteOrganizationRoleBaseRequest(_message.Message):
    __slots__ = ("org_id", "role_name")
    ORG_ID_FIELD_NUMBER: _ClassVar[int]
    ROLE_NAME_FIELD_NUMBER: _ClassVar[int]
    org_id: str
    role_name: str
    def __init__(self, org_id: _Optional[str] = ..., role_name: _Optional[str] = ...) -> None: ...



from typing import Optional, Dict, Any

from  ..tool_input_output import ToolOutput
from pydantic import BaseModel, Field
from google.protobuf.json_format import MessageToDict



class ExecuteToolResponse(BaseModel):
    """Execute tool response with one-to-one mapping to proto ExecuteToolResponse"""
    
    data: Optional[ToolOutput] = Field(
        None,
        description="Free-flowing JSON parameters for the tool execution"
    )
    execution_id: Optional[str] = Field(
        None,
        description="Unique identifier for the tool execution"
    )

    @classmethod
    def from_proto(cls, proto_response) -> 'ExecuteToolResponse':
        """
        Create ExecuteToolResponse from protobuf ExecuteToolResponse
        
        :param proto_response: The protobuf ExecuteToolResponse object
        :type proto_response: ExecuteToolResponse (from tools_pb2)
        
        :returns:
            ExecuteToolResponse instance
        """
        # Convert protobuf Struct to dict
        data = None
        if proto_response.data:
            data = MessageToDict(proto_response.data)

        return cls(
            data=data,
            execution_id=proto_response.execution_id if proto_response.execution_id else None
        )

    def to_dict(self) -> Dict[str, Any]:
        """
        Convert to dictionary representation
        
        :returns:
            Dictionary representation of the response
        """
        return {
            "data": self.data,
            "execution_id": self.execution_id
        }

    class Config:
        """Pydantic configuration"""
        validate_assignment = True
        arbitrary_types_allowed = True


from typing import TypeVar, Optional, Protocol

import grpc
import jwt
import json
import requests
import platform
from urllib.parse import urlparse

from cryptography.hazmat.primitives import serialization
from scalekit.common.scalekit import GrantType
from scalekit.common.exceptions import ScalekitServerException, ScalekitException

TRequest = TypeVar("TRequest")
TResponse = TypeVar("TResponse")
TMetadata = TypeVar("TMetadata")

TOKEN_ENDPOINT = "/oauth/token"
JWKS_ENDPOINT = "/keys"


class WithCall(Protocol):
    def __call__(self, request: TRequest, metadata: TMetadata) -> TResponse: ...


class CoreClient:
    """Class definition for Core Client"""

    sdk_version = "Scalekit-Python/2.3.2"
    api_version = "20250718"
    user_agent = f"{sdk_version} Python/{platform.python_version()} ({platform.system()}; {platform.architecture()}"

    def __init__(self, env_url, client_id, client_secret):
        """
        Initializer for Core client

        :param env_url        : Environment URL
        :type                 : ``` str ```
        :param client_id      : Client ID
        :type                 : ``` str ```
        :param client_secret  : Client Secret
        :type                 : ``` str ```
        :returns
            None
        """
        parsed_url = urlparse(env_url)
        self.host = parsed_url.netloc
        self.env_url = env_url
        self.client_id = client_id
        self.client_secret = client_secret
        self.keys = {}
        self.access_token = None
        self.grpc_secure_channel = None
        self.__authenticate_client()
        self.__grpc_secure_channel()

    def __grpc_secure_channel(self):
        """
        Method to authenticate grpc and create secure grpc channel
        :params
            None
        :returns
            None
        """
        channel_credentials = grpc.ssl_channel_credentials()
        call_credentials = grpc.access_token_call_credentials(self.access_token)
        composite_credentials = grpc.composite_channel_credentials(
            channel_credentials,
            call_credentials,
        )
        self.grpc_secure_channel = grpc.secure_channel(self.host, composite_credentials)

    def __authenticate_client(self):
        """
        Method to authenticate client  and return access token

        :returns
            access_token
        """
        params = {
            "grant_type": GrantType.ClientCredentials.value,
            "client_id": self.client_id,
            "client_secret": self.client_secret,
        }

        response = self.authenticate(data=json.dumps(params))
        if response.status_code != 200:
            raise ScalekitServerException.promote(response)
        response = json.loads(response.content)
        self.access_token = response["access_token"]

    def authenticate(self, data: str):
        """
        Method to execute post request for authentication with given user params

        :param data : params for authentication request
        :type       : ``` str ```
        """
        headers = {"Content-Type": "application/x-www-form-urlencoded"}
        response = requests.post(
            self.env_url + TOKEN_ENDPOINT,
            headers=self.get_headers(headers=headers),
            data=json.loads(data),
            verify=True,
        )
        if response.status_code != 200:
            raise ScalekitServerException.promote(response)
        return response

    def get_jwks(self):
        """Method to get JWT Keys"""
        if self.keys and len(self.keys) > 0:
            return
        response = requests.get(
            self.env_url + JWKS_ENDPOINT, headers=self.get_headers()
        )
        response = json.loads(response.content)
        keys = response["keys"]

        for key in keys:
            kid = key["kid"]
            rsa_key = jwt.PyJWK.from_dict(key).key

            pem_key = rsa_key.public_bytes(
                encoding=serialization.Encoding.PEM,
                format=serialization.PublicFormat.SubjectPublicKeyInfo
            )

            self.keys[kid] = pem_key.decode("utf-8")

    def get_headers(self, headers: Optional[dict] = None) -> dict:
        """
        Method to get user defined headers and returns collated header params

        :param headers : User defined header dictionary
        :type          : ``` dict ```
        :returns
            dict
        """
        default_headers = {
            "user-agent": f"{self.user_agent}",
            "x-api-version": f"{self.api_version}",
            "x-sdk-version": f"{self.sdk_version}",
        }
        if self.access_token:
            default_headers.update({"authorization": f"Bearer {self.access_token}"})
        if headers:
            return {**default_headers, **headers}
        return default_headers

    def grpc_exec(
        self,
        func: WithCall,
        data: TRequest,
        retry=1,
    ) -> TResponse:
        try:
            resp = func(
                data,
                metadata=tuple(self.get_headers().items()),
            )
            return resp
        except grpc.RpcError as exp:
            if retry > 0:
                return self.grpc_exec(func, data, retry=retry - 1)
            else:
                raise ScalekitServerException.promote(exp)
        except Exception as exp:
            raise ScalekitException(exp)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: scalekit/v1/auth/passwordless.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from protoc_gen_openapiv2.options import annotations_pb2 as protoc__gen__openapiv2_dot_options_dot_annotations__pb2
from scalekit.v1.commons import commons_pb2 as scalekit_dot_v1_dot_commons_dot_commons__pb2
from scalekit.v1.options import options_pb2 as scalekit_dot_v1_dot_options_dot_options__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#scalekit/v1/auth/passwordless.proto\x12\x1dscalekit.v1.auth.passwordless\x1a\x1b\x62uf/validate/validate.proto\x1a\x1cgoogle/api/annotations.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a.protoc-gen-openapiv2/options/annotations.proto\x1a!scalekit/v1/commons/commons.proto\x1a!scalekit/v1/options/options.proto\"\xc8\x10\n\x17SendPasswordlessRequest\x12\xad\x01\n\x05\x65mail\x18\x01 \x01(\tB\x96\x01\x92\x41\x85\x01\x32kEmail address where the passwordless authentication credentials will be sent. Must be a valid email format.J\x16\"john.doe@example.com\"\xbaH\nr\x05\x10\x01\x18\xc0\x02\xc8\x01\x01R\x05\x65mail\x12\xa6\x02\n\x08template\x18\x02 \x01(\x0e\x32+.scalekit.v1.auth.passwordless.TemplateTypeB\xd7\x01\x92\x41\xcb\x01\x32\xbe\x01Specifies the authentication intent for the passwordless request. Use SIGNIN for existing users or SIGNUP for new user registration. This affects the email template and user experience flow.J\x08\"SIGNIN\"\xbaH\x05\x82\x01\x02\x10\x01H\x00R\x08template\x88\x01\x01\x12\xa9\x02\n\x12magiclink_auth_uri\x18\x03 \x01(\tB\xf5\x01\x92\x41\xe4\x01\x32\xaf\x01Your application\'s callback URL where users will be redirected after clicking the magic link in their email. The link token will be appended as a query parameter as link_tokenJ0\"https://yourapp.com/auth/passwordless/callback\"\xe0\x41\x01\xbaH\x07r\x05\x10\x01\x18\xe8\x07H\x01R\x10magiclinkAuthUri\x88\x01\x01\x12\x9b\x02\n\x05state\x18\x04 \x01(\tB\xff\x01\x92\x41\xee\x01\x32\xd9\x01\x43ustom state parameter that will be returned unchanged in the verification response. Use this to maintain application state between the authentication request and callback, such as the intended destination after loginJ\x10\"d62ivasry29lso\"\xe0\x41\x01\xbaH\x07r\x05\x10\x01\x18\xe8\x07H\x02R\x05state\x88\x01\x01\x12\xa6\x01\n\nexpires_in\x18\x05 \x01(\rB\x81\x01\x92\x41{2tTime in seconds until the passwordless authentication expires. If not specified, defaults to 300 seconds (5 minutes)J\x03\x33\x30\x30\xe0\x41\x01H\x03R\texpiresIn\x88\x01\x01\x12\xdd\x05\n\x12template_variables\x18\x06 \x03(\x0b\x32M.scalekit.v1.auth.passwordless.SendPasswordlessRequest.TemplateVariablesEntryB\xde\x04\x92\x41\xcf\x04\x32\x9a\x04\x41 set of key-value pairs to personalize the email template.\n\n* You may include up to 30 key-value pairs.\n* The following variable names are reserved by the system and cannot be supplied: `otp`, `expiry_time_relative`, `link`, `expire_time`, `expiry_time`.\n* Every variable referenced in your email template must be included as a key-value pair.\n\nUse these variables to insert custom information, such as a team name, URL or the user\'s employee ID. All variables are interpolated before the email is sent, regardless of the email provider.J0{\"custom_variable_key\": \"custom_variable_value\"}\xe0\x41\x01\xbaH\x05\x9a\x01\x02\x10\x1eR\x11templateVariables\x1a\x44\n\x16TemplateVariablesEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\x0b\n\t_templateB\x15\n\x13_magiclink_auth_uriB\x08\n\x06_stateB\r\n\x0b_expires_in\"\x9c\x07\n\x18SendPasswordlessResponse\x12\xc5\x01\n\x0f\x61uth_request_id\x18\x01 \x01(\tB\x9c\x01\x92\x41\x95\x01\x32]Unique identifier for this passwordless authentication request. Use this ID to resend emails.J4\"h5Y8kT5RVwaea5WEgW4n-6C-aO_-fuTUW7Vb9-Rh3AcY9qxZqQ\"\xe0\x41\x03R\rauthRequestId\x12\xcc\x01\n\nexpires_at\x18\x02 \x01(\x03\x42\xac\x01\x92\x41\xa5\x01\x32\x96\x01Unix timestamp (seconds since epoch) when the passwordless authentication will expire. After this time, the OTP or magic link will no longer be valid.J\n1748696575\xe0\x41\x03R\texpiresAt\x12\xbe\x01\n\nexpires_in\x18\x03 \x01(\rB\x9e\x01\x92\x41\x97\x01\x32\x8f\x01Number of seconds from now until the passwordless authentication expires. This is a convenience field calculated from the expires_at timestamp.J\x03\x33\x30\x30\xe0\x41\x03R\texpiresIn\x12\xa7\x02\n\x11passwordless_type\x18\x04 \x01(\x0e\x32/.scalekit.v1.auth.passwordless.PasswordlessTypeB\xc8\x01\x92\x41\xbc\x01\x32\xb2\x01Type of passwordless authentication that was sent via email. OTP sends a numeric code, LINK sends a clickable magic link, and LINK_OTP provides both options for user convenience.J\x05\"OTP\"\xbaH\x05\x82\x01\x02\x10\x01R\x10passwordlessType\"\xb4\x06\n\x19VerifyPasswordLessRequest\x12\xbd\x01\n\x04\x63ode\x18\x01 \x01(\tB\xa6\x01\x92\x41\x99\x01\x32\x8c\x01The Verification Code (OTP) received via email. This is typically a 6-digit numeric code that users enter manually to verify their identity.J\x08\"123456\"\xbaH\x06r\x04\x10\x01\x18\x14H\x00R\x04\x63ode\x12\x8a\x02\n\nlink_token\x18\x02 \x01(\tB\xe8\x01\x92\x41\xdb\x01\x32\xb0\x01The unique token from the magic link URL received via email. Extract this token when users click the magic link and are redirected to your application to later verify the user.J&\"afe9d61c-d80d-4020-a8ee-61765ab71cb3\"\xbaH\x06r\x04\x10\x01\x18\x64H\x00R\tlinkToken\x12\xa2\x02\n\x0f\x61uth_request_id\x18\x03 \x01(\tB\xf4\x01\x92\x41\xe8\x01\x32\xaf\x01The authentication request identifier returned from the send passwordless email endpoint. Required when verifying OTP codes to link the verification with the original request.J4\"h5Y8kT5RVwaea5WEgW4n-6C-aO_-fuTUW7Vb9-Rh3AcY9qxZqQ\"\xbaH\x05r\x03\x18\xc8\x01H\x01R\rauthRequestId\x88\x01\x01\x42\x11\n\x0f\x61uth_credentialB\x12\n\x10_auth_request_id\"\xc1\x02\n\x19ResendPasswordlessRequest\x12\xa3\x02\n\x0f\x61uth_request_id\x18\x01 \x01(\tB\xfa\x01\x92\x41\xe9\x01\x32\xb0\x01The authentication request identifier from the original send passwordless email request. Use this to resend the Verification Code (OTP) or Magic Link to the same email address.J4\"h5Y8kT5RVwaea5WEgW4n-6C-aO_-fuTUW7Vb9-Rh3AcY9qxZqQ\"\xbaH\nr\x05\x10\x01\x18\xc8\x01\xc8\x01\x01R\rauthRequestId\"\x94\x07\n\x1aVerifyPasswordLessResponse\x12\xbc\x01\n\x05\x65mail\x18\x01 \x01(\tB\xa5\x01\x92\x41\x9e\x01\x32\x83\x01\x45mail address of the successfully authenticated user. This confirms which email account was verified through the passwordless flow.J\x16\"john.doe@example.com\"\xe0\x41\x03R\x05\x65mail\x12\xe8\x01\n\x05state\x18\x02 \x01(\tB\xcc\x01\x92\x41\xc5\x01\x32\xad\x01The custom state parameter that was provided in the original authentication request, returned unchanged. Use this to restore your application\'s context after authentication.J\x13\"kdt7yiag28t341fr1\"\xe0\x41\x03H\x00R\x05state\x88\x01\x01\x12\xcb\x01\n\x08template\x18\x03 \x01(\x0e\x32+.scalekit.v1.auth.passwordless.TemplateTypeB}\x92\x41r2fSpecifies which email template to choose. For User Signin choose SIGNIN and for User Signup use SIGNUPJ\x08\"SIGNIN\"\xbaH\x05\x82\x01\x02\x10\x01H\x01R\x08template\x88\x01\x01\x12\xe6\x01\n\x11passwordless_type\x18\x04 \x01(\x0e\x32/.scalekit.v1.auth.passwordless.PasswordlessTypeB\x87\x01\x92\x41|2sThe type of passwordless authentication that was successfully verified, confirming which method the user completed.J\x05\"OTP\"\xbaH\x05\x82\x01\x02\x10\x01R\x10passwordlessTypeB\x08\n\x06_stateB\x0b\n\t_template*7\n\x0cTemplateType\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\n\n\x06SIGNIN\x10\x01\x12\n\n\x06SIGNUP\x10\x02*V\n\x10PasswordlessType\x12!\n\x1dPASSWORDLESS_TYPE_UNSPECIFIED\x10\x00\x12\x07\n\x03OTP\x10\x01\x12\x08\n\x04LINK\x10\x02\x12\x0c\n\x08LINK_OTP\x10\x03\x32\xe9!\n\x13PasswordlessService\x12\xdb\x0c\n\x15SendPasswordlessEmail\x12\x36.scalekit.v1.auth.passwordless.SendPasswordlessRequest\x1a\x37.scalekit.v1.auth.passwordless.SendPasswordlessResponse\"\xd0\x0b\x92\x41\x9c\x0b\n\x11Passwordless Auth\x12\x17Send passwordless email\x1atSend a verification email containing either a verification code (OTP), magic link, or both to a user\'s email addressJ\xd0\x01\n\x03\x32\x30\x30\x12\xc8\x01\n\x8d\x01Successfully sent passwordless authentication email. Returns the authentication request details including expiration time and auth request ID\x12\x36\n4\x1a\x32#/definitions/passwordlessSendPasswordlessResponsej\xa4\x08\n\rx-codeSamples\x12\x92\x08\x32\x8f\x08\n\xfc\x02*\xf9\x02\n\x16\n\x05label\x12\r\x1a\x0bNode.js SDK\n\x14\n\x04lang\x12\x0c\x1a\njavascript\n\xc8\x02\n\x06source\x12\xbd\x02\x1a\xba\x02\x63onst response = await scalekit.passwordless.\n  sendPasswordlessEmail(\n\t\"john.doe@example.com\",\n\t{\n\t\ttemplate: \"SIGNIN\",\n\t\texpiresIn: 100,\n\t\tmagiclinkAuthUri: \"https://www.google.com\",\n\t\ttemplateVariables: {\n\t\t\temployeeID: \"EMP523\",\n\t\t\tteamName: \"Alpha Team\",\n\t\t\tsupportEmail: \"support@yourcompany.com\",\n\t\t},\n\t}\n);\n\x8d\x05*\x8a\x05\n\x11\n\x05label\x12\x08\x1a\x06Go SDK\n\x0c\n\x04lang\x12\x04\x1a\x02go\n\xe6\x04\n\x06source\x12\xdb\x04\x1a\xd8\x04templateType := scalekit.TemplateTypeSignin\nresponse, err := client.Passwordless().SendPasswordlessEmail(\n    ctx,\n    \"john.doe@example.com\",\n    &scalekit.SendPasswordlessOptions{\n        Template:         &templateType,\n        ExpiresIn:        100,\n        MagiclinkAuthUri: \"https://www.google.com\",\n        TemplateVariables: map[string]string{\n            \"employeeID\":    \"EMP523\",\n            \"teamName\":      \"Alpha Team\",\n            \"supportEmail\":  \"support@yourcompany.com\",\n        },\n    },\n)\n\nif err != nil {\n    // Handle error\n    return\n}\n\nauthRequestId := response.AuthRequestId\x82\xb5\x18\x02\x18\x04\x82\xd3\xe4\x93\x02$\"\x1f/api/v1/passwordless/email/send:\x01*\x12\xbc\x0b\n\x17VerifyPasswordlessEmail\x12\x38.scalekit.v1.auth.passwordless.VerifyPasswordLessRequest\x1a\x39.scalekit.v1.auth.passwordless.VerifyPasswordLessResponse\"\xab\n\x92\x41\xf5\t\n\x11Passwordless Auth\x12\x19Verify passwordless email\x1aMVerify a user\'s identity using either a verification code or magic link tokenJ\x92\x01\n\x03\x32\x30\x30\x12\x8a\x01\nISuccessfully verified the passwordless authentication. Returns user email\x12=\n;\x1a\x39.scalekit.v1.auth.passwordless.VerifyPasswordLessResponsej\xe0\x07\n\rx-codeSamples\x12\xce\x07\x32\xcb\x07\n\xac\x02*\xa9\x02\n\x16\n\x05label\x12\r\x1a\x0bNode.js SDK\n\x14\n\x04lang\x12\x0c\x1a\njavascript\n\xf8\x01\n\x06source\x12\xed\x01\x1a\xea\x01\x63onst { authRequestId } = sendResponse;\nconst verifyResponse = await scalekit.passwordless.\n  verifyPasswordlessEmail(\n\t// Verification Code (OTP)\n\t{ code: \"123456\" },\n\t// Magic Link Token\n\t{ linkToken: link_token },\n\tauthRequestId\n);\n\x99\x05*\x96\x05\n\x11\n\x05label\x12\x08\x1a\x06Go SDK\n\x0c\n\x04lang\x12\x04\x1a\x02go\n\xf2\x04\n\x06source\x12\xe7\x04\x1a\xe4\x04// Verify with OTP code\nverifyResponse, err := client.Passwordless().VerifyPasswordlessEmail(\n    ctx,\n    &scalekit.VerifyPasswordlessOptions{\n        Code:          \"123456\", // OTP code\n        AuthRequestId: authRequestId,\n    },\n)\n\nif err != nil {\n    // Handle error\n    return\n}\n\n// Verify with magic link token\nverifyResponse, err := client.Passwordless().VerifyPasswordlessEmail(\n    ctx,\n    &scalekit.VerifyPasswordlessOptions{\n        LinkToken: linkToken, // Magic link token\n    },\n)\n\nif err != nil {\n    // Handle error\n    return\n}\n\n// User verified successfully\nuserEmail := verifyResponse.Email\x82\xb5\x18\x02\x18\x04\x82\xd3\xe4\x93\x02&\"!/api/v1/passwordless/email/verify:\x01*\x12\xd8\x07\n\x17ResendPasswordlessEmail\x12\x38.scalekit.v1.auth.passwordless.ResendPasswordlessRequest\x1a\x37.scalekit.v1.auth.passwordless.SendPasswordlessResponse\"\xc9\x06\x92\x41\x93\x06\n\x11Passwordless Auth\x12\x19Resend passwordless email\x1a\x62Resend a verification email if the user didn\'t receive it or if the previous code/link has expiredJ\xcb\x01\n\x03\x32\x30\x30\x12\xc3\x01\n\x83\x01Successfully resent the passwordless authentication email. Returns updated authentication request details with new expiration time.\x12;\n9\x1a\x37.scalekit.v1.auth.passwordless.SendPasswordlessResponsej\xb0\x03\n\rx-codeSamples\x12\x9e\x03\x32\x9b\x03\n\xcb\x01*\xc8\x01\n\x16\n\x05label\x12\r\x1a\x0bNode.js SDK\n\x14\n\x04lang\x12\x0c\x1a\njavascript\n\x97\x01\n\x06source\x12\x8c\x01\x1a\x89\x01\x63onst { authRequestId } = sendResponse;\nconst resendResponse = await scalekit.passwordless\n.resendPasswordlessEmail(\n    authRequestId\n);\n\xca\x01*\xc7\x01\n\x11\n\x05label\x12\x08\x1a\x06Go SDK\n\x0c\n\x04lang\x12\x04\x1a\x02go\n\xa3\x01\n\x06source\x12\x98\x01\x1a\x95\x01resendResponse, err := client.Passwordless().ResendPasswordlessEmail(\n    ctx,\n    authRequestId,\n)\n\nif err != nil {\n    // Handle error\n    return\n}\x82\xb5\x18\x02\x18\x04\x82\xd3\xe4\x93\x02&\"!/api/v1/passwordless/email/resend:\x01*\x1a\xd9\x01\x92\x41\xd5\x01\n\x11Passwordless Auth\x12\xbf\x01\x45ndpoints for sending and verifying passwordless authentication emails. These APIs allow users to authenticate without passwords by receiving a verification code or magic link in their email.B=Z;github.com/scalekit-inc/scalekit/pkg/grpc/auth/passwordlessb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'scalekit.v1.auth.passwordless_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z;github.com/scalekit-inc/scalekit/pkg/grpc/auth/passwordless'
  _globals['_SENDPASSWORDLESSREQUEST_TEMPLATEVARIABLESENTRY']._loaded_options = None
  _globals['_SENDPASSWORDLESSREQUEST_TEMPLATEVARIABLESENTRY']._serialized_options = b'8\001'
  _globals['_SENDPASSWORDLESSREQUEST'].fields_by_name['email']._loaded_options = None
  _globals['_SENDPASSWORDLESSREQUEST'].fields_by_name['email']._serialized_options = b'\222A\205\0012kEmail address where the passwordless authentication credentials will be sent. Must be a valid email format.J\026\"john.doe@example.com\"\272H\nr\005\020\001\030\300\002\310\001\001'
  _globals['_SENDPASSWORDLESSREQUEST'].fields_by_name['template']._loaded_options = None
  _globals['_SENDPASSWORDLESSREQUEST'].fields_by_name['template']._serialized_options = b'\222A\313\0012\276\001Specifies the authentication intent for the passwordless request. Use SIGNIN for existing users or SIGNUP for new user registration. This affects the email template and user experience flow.J\010\"SIGNIN\"\272H\005\202\001\002\020\001'
  _globals['_SENDPASSWORDLESSREQUEST'].fields_by_name['magiclink_auth_uri']._loaded_options = None
  _globals['_SENDPASSWORDLESSREQUEST'].fields_by_name['magiclink_auth_uri']._serialized_options = b'\222A\344\0012\257\001Your application\'s callback URL where users will be redirected after clicking the magic link in their email. The link token will be appended as a query parameter as link_tokenJ0\"https://yourapp.com/auth/passwordless/callback\"\340A\001\272H\007r\005\020\001\030\350\007'
  _globals['_SENDPASSWORDLESSREQUEST'].fields_by_name['state']._loaded_options = None
  _globals['_SENDPASSWORDLESSREQUEST'].fields_by_name['state']._serialized_options = b'\222A\356\0012\331\001Custom state parameter that will be returned unchanged in the verification response. Use this to maintain application state between the authentication request and callback, such as the intended destination after loginJ\020\"d62ivasry29lso\"\340A\001\272H\007r\005\020\001\030\350\007'
  _globals['_SENDPASSWORDLESSREQUEST'].fields_by_name['expires_in']._loaded_options = None
  _globals['_SENDPASSWORDLESSREQUEST'].fields_by_name['expires_in']._serialized_options = b'\222A{2tTime in seconds until the passwordless authentication expires. If not specified, defaults to 300 seconds (5 minutes)J\003300\340A\001'
  _globals['_SENDPASSWORDLESSREQUEST'].fields_by_name['template_variables']._loaded_options = None
  _globals['_SENDPASSWORDLESSREQUEST'].fields_by_name['template_variables']._serialized_options = b'\222A\317\0042\232\004A set of key-value pairs to personalize the email template.\n\n* You may include up to 30 key-value pairs.\n* The following variable names are reserved by the system and cannot be supplied: `otp`, `expiry_time_relative`, `link`, `expire_time`, `expiry_time`.\n* Every variable referenced in your email template must be included as a key-value pair.\n\nUse these variables to insert custom information, such as a team name, URL or the user\'s employee ID. All variables are interpolated before the email is sent, regardless of the email provider.J0{\"custom_variable_key\": \"custom_variable_value\"}\340A\001\272H\005\232\001\002\020\036'
  _globals['_SENDPASSWORDLESSRESPONSE'].fields_by_name['auth_request_id']._loaded_options = None
  _globals['_SENDPASSWORDLESSRESPONSE'].fields_by_name['auth_request_id']._serialized_options = b'\222A\225\0012]Unique identifier for this passwordless authentication request. Use this ID to resend emails.J4\"h5Y8kT5RVwaea5WEgW4n-6C-aO_-fuTUW7Vb9-Rh3AcY9qxZqQ\"\340A\003'
  _globals['_SENDPASSWORDLESSRESPONSE'].fields_by_name['expires_at']._loaded_options = None
  _globals['_SENDPASSWORDLESSRESPONSE'].fields_by_name['expires_at']._serialized_options = b'\222A\245\0012\226\001Unix timestamp (seconds since epoch) when the passwordless authentication will expire. After this time, the OTP or magic link will no longer be valid.J\n1748696575\340A\003'
  _globals['_SENDPASSWORDLESSRESPONSE'].fields_by_name['expires_in']._loaded_options = None
  _globals['_SENDPASSWORDLESSRESPONSE'].fields_by_name['expires_in']._serialized_options = b'\222A\227\0012\217\001Number of seconds from now until the passwordless authentication expires. This is a convenience field calculated from the expires_at timestamp.J\003300\340A\003'
  _globals['_SENDPASSWORDLESSRESPONSE'].fields_by_name['passwordless_type']._loaded_options = None
  _globals['_SENDPASSWORDLESSRESPONSE'].fields_by_name['passwordless_type']._serialized_options = b'\222A\274\0012\262\001Type of passwordless authentication that was sent via email. OTP sends a numeric code, LINK sends a clickable magic link, and LINK_OTP provides both options for user convenience.J\005\"OTP\"\272H\005\202\001\002\020\001'
  _globals['_VERIFYPASSWORDLESSREQUEST'].fields_by_name['code']._loaded_options = None
  _globals['_VERIFYPASSWORDLESSREQUEST'].fields_by_name['code']._serialized_options = b'\222A\231\0012\214\001The Verification Code (OTP) received via email. This is typically a 6-digit numeric code that users enter manually to verify their identity.J\010\"123456\"\272H\006r\004\020\001\030\024'
  _globals['_VERIFYPASSWORDLESSREQUEST'].fields_by_name['link_token']._loaded_options = None
  _globals['_VERIFYPASSWORDLESSREQUEST'].fields_by_name['link_token']._serialized_options = b'\222A\333\0012\260\001The unique token from the magic link URL received via email. Extract this token when users click the magic link and are redirected to your application to later verify the user.J&\"afe9d61c-d80d-4020-a8ee-61765ab71cb3\"\272H\006r\004\020\001\030d'
  _globals['_VERIFYPASSWORDLESSREQUEST'].fields_by_name['auth_request_id']._loaded_options = None
  _globals['_VERIFYPASSWORDLESSREQUEST'].fields_by_name['auth_request_id']._serialized_options = b'\222A\350\0012\257\001The authentication request identifier returned from the send passwordless email endpoint. Required when verifying OTP codes to link the verification with the original request.J4\"h5Y8kT5RVwaea5WEgW4n-6C-aO_-fuTUW7Vb9-Rh3AcY9qxZqQ\"\272H\005r\003\030\310\001'
  _globals['_RESENDPASSWORDLESSREQUEST'].fields_by_name['auth_request_id']._loaded_options = None
  _globals['_RESENDPASSWORDLESSREQUEST'].fields_by_name['auth_request_id']._serialized_options = b'\222A\351\0012\260\001The authentication request identifier from the original send passwordless email request. Use this to resend the Verification Code (OTP) or Magic Link to the same email address.J4\"h5Y8kT5RVwaea5WEgW4n-6C-aO_-fuTUW7Vb9-Rh3AcY9qxZqQ\"\272H\nr\005\020\001\030\310\001\310\001\001'
  _globals['_VERIFYPASSWORDLESSRESPONSE'].fields_by_name['email']._loaded_options = None
  _globals['_VERIFYPASSWORDLESSRESPONSE'].fields_by_name['email']._serialized_options = b'\222A\236\0012\203\001Email address of the successfully authenticated user. This confirms which email account was verified through the passwordless flow.J\026\"john.doe@example.com\"\340A\003'
  _globals['_VERIFYPASSWORDLESSRESPONSE'].fields_by_name['state']._loaded_options = None
  _globals['_VERIFYPASSWORDLESSRESPONSE'].fields_by_name['state']._serialized_options = b'\222A\305\0012\255\001The custom state parameter that was provided in the original authentication request, returned unchanged. Use this to restore your application\'s context after authentication.J\023\"kdt7yiag28t341fr1\"\340A\003'
  _globals['_VERIFYPASSWORDLESSRESPONSE'].fields_by_name['template']._loaded_options = None
  _globals['_VERIFYPASSWORDLESSRESPONSE'].fields_by_name['template']._serialized_options = b'\222Ar2fSpecifies which email template to choose. For User Signin choose SIGNIN and for User Signup use SIGNUPJ\010\"SIGNIN\"\272H\005\202\001\002\020\001'
  _globals['_VERIFYPASSWORDLESSRESPONSE'].fields_by_name['passwordless_type']._loaded_options = None
  _globals['_VERIFYPASSWORDLESSRESPONSE'].fields_by_name['passwordless_type']._serialized_options = b'\222A|2sThe type of passwordless authentication that was successfully verified, confirming which method the user completed.J\005\"OTP\"\272H\005\202\001\002\020\001'
  _globals['_PASSWORDLESSSERVICE']._loaded_options = None
  _globals['_PASSWORDLESSSERVICE']._serialized_options = b'\222A\325\001\n\021Passwordless Auth\022\277\001Endpoints for sending and verifying passwordless authentication emails. These APIs allow users to authenticate without passwords by receiving a verification code or magic link in their email.'
  _globals['_PASSWORDLESSSERVICE'].methods_by_name['SendPasswordlessEmail']._loaded_options = None
  _globals['_PASSWORDLESSSERVICE'].methods_by_name['SendPasswordlessEmail']._serialized_options = b'\222A\234\013\n\021Passwordless Auth\022\027Send passwordless email\032tSend a verification email containing either a verification code (OTP), magic link, or both to a user\'s email addressJ\320\001\n\003200\022\310\001\n\215\001Successfully sent passwordless authentication email. Returns the authentication request details including expiration time and auth request ID\0226\n4\0322#/definitions/passwordlessSendPasswordlessResponsej\244\010\n\rx-codeSamples\022\222\0102\217\010\n\374\002*\371\002\n\026\n\005label\022\r\032\013Node.js SDK\n\024\n\004lang\022\014\032\njavascript\n\310\002\n\006source\022\275\002\032\272\002const response = await scalekit.passwordless.\n  sendPasswordlessEmail(\n\t\"john.doe@example.com\",\n\t{\n\t\ttemplate: \"SIGNIN\",\n\t\texpiresIn: 100,\n\t\tmagiclinkAuthUri: \"https://www.google.com\",\n\t\ttemplateVariables: {\n\t\t\temployeeID: \"EMP523\",\n\t\t\tteamName: \"Alpha Team\",\n\t\t\tsupportEmail: \"support@yourcompany.com\",\n\t\t},\n\t}\n);\n\215\005*\212\005\n\021\n\005label\022\010\032\006Go SDK\n\014\n\004lang\022\004\032\002go\n\346\004\n\006source\022\333\004\032\330\004templateType := scalekit.TemplateTypeSignin\nresponse, err := client.Passwordless().SendPasswordlessEmail(\n    ctx,\n    \"john.doe@example.com\",\n    &scalekit.SendPasswordlessOptions{\n        Template:         &templateType,\n        ExpiresIn:        100,\n        MagiclinkAuthUri: \"https://www.google.com\",\n        TemplateVariables: map[string]string{\n            \"employeeID\":    \"EMP523\",\n            \"teamName\":      \"Alpha Team\",\n            \"supportEmail\":  \"support@yourcompany.com\",\n        },\n    },\n)\n\nif err != nil {\n    // Handle error\n    return\n}\n\nauthRequestId := response.AuthRequestId\202\265\030\002\030\004\202\323\344\223\002$\"\037/api/v1/passwordless/email/send:\001*'
  _globals['_PASSWORDLESSSERVICE'].methods_by_name['VerifyPasswordlessEmail']._loaded_options = None
  _globals['_PASSWORDLESSSERVICE'].methods_by_name['VerifyPasswordlessEmail']._serialized_options = b'\222A\365\t\n\021Passwordless Auth\022\031Verify passwordless email\032MVerify a user\'s identity using either a verification code or magic link tokenJ\222\001\n\003200\022\212\001\nISuccessfully verified the passwordless authentication. Returns user email\022=\n;\0329.scalekit.v1.auth.passwordless.VerifyPasswordLessResponsej\340\007\n\rx-codeSamples\022\316\0072\313\007\n\254\002*\251\002\n\026\n\005label\022\r\032\013Node.js SDK\n\024\n\004lang\022\014\032\njavascript\n\370\001\n\006source\022\355\001\032\352\001const { authRequestId } = sendResponse;\nconst verifyResponse = await scalekit.passwordless.\n  verifyPasswordlessEmail(\n\t// Verification Code (OTP)\n\t{ code: \"123456\" },\n\t// Magic Link Token\n\t{ linkToken: link_token },\n\tauthRequestId\n);\n\231\005*\226\005\n\021\n\005label\022\010\032\006Go SDK\n\014\n\004lang\022\004\032\002go\n\362\004\n\006source\022\347\004\032\344\004// Verify with OTP code\nverifyResponse, err := client.Passwordless().VerifyPasswordlessEmail(\n    ctx,\n    &scalekit.VerifyPasswordlessOptions{\n        Code:          \"123456\", // OTP code\n        AuthRequestId: authRequestId,\n    },\n)\n\nif err != nil {\n    // Handle error\n    return\n}\n\n// Verify with magic link token\nverifyResponse, err := client.Passwordless().VerifyPasswordlessEmail(\n    ctx,\n    &scalekit.VerifyPasswordlessOptions{\n        LinkToken: linkToken, // Magic link token\n    },\n)\n\nif err != nil {\n    // Handle error\n    return\n}\n\n// User verified successfully\nuserEmail := verifyResponse.Email\202\265\030\002\030\004\202\323\344\223\002&\"!/api/v1/passwordless/email/verify:\001*'
  _globals['_PASSWORDLESSSERVICE'].methods_by_name['ResendPasswordlessEmail']._loaded_options = None
  _globals['_PASSWORDLESSSERVICE'].methods_by_name['ResendPasswordlessEmail']._serialized_options = b'\222A\223\006\n\021Passwordless Auth\022\031Resend passwordless email\032bResend a verification email if the user didn\'t receive it or if the previous code/link has expiredJ\313\001\n\003200\022\303\001\n\203\001Successfully resent the passwordless authentication email. Returns updated authentication request details with new expiration time.\022;\n9\0327.scalekit.v1.auth.passwordless.SendPasswordlessResponsej\260\003\n\rx-codeSamples\022\236\0032\233\003\n\313\001*\310\001\n\026\n\005label\022\r\032\013Node.js SDK\n\024\n\004lang\022\014\032\njavascript\n\227\001\n\006source\022\214\001\032\211\001const { authRequestId } = sendResponse;\nconst resendResponse = await scalekit.passwordless\n.resendPasswordlessEmail(\n    authRequestId\n);\n\312\001*\307\001\n\021\n\005label\022\010\032\006Go SDK\n\014\n\004lang\022\004\032\002go\n\243\001\n\006source\022\230\001\032\225\001resendResponse, err := client.Passwordless().ResendPasswordlessEmail(\n    ctx,\n    authRequestId,\n)\n\nif err != nil {\n    // Handle error\n    return\n}\202\265\030\002\030\004\202\323\344\223\002&\"!/api/v1/passwordless/email/resend:\001*'
  _globals['_TEMPLATETYPE']._serialized_start=5488
  _globals['_TEMPLATETYPE']._serialized_end=5543
  _globals['_PASSWORDLESSTYPE']._serialized_start=5545
  _globals['_PASSWORDLESSTYPE']._serialized_end=5631
  _globals['_SENDPASSWORDLESSREQUEST']._serialized_start=373
  _globals['_SENDPASSWORDLESSREQUEST']._serialized_end=2493
  _globals['_SENDPASSWORDLESSREQUEST_TEMPLATEVARIABLESENTRY']._serialized_start=2364
  _globals['_SENDPASSWORDLESSREQUEST_TEMPLATEVARIABLESENTRY']._serialized_end=2432
  _globals['_SENDPASSWORDLESSRESPONSE']._serialized_start=2496
  _globals['_SENDPASSWORDLESSRESPONSE']._serialized_end=3420
  _globals['_VERIFYPASSWORDLESSREQUEST']._serialized_start=3423
  _globals['_VERIFYPASSWORDLESSREQUEST']._serialized_end=4243
  _globals['_RESENDPASSWORDLESSREQUEST']._serialized_start=4246
  _globals['_RESENDPASSWORDLESSREQUEST']._serialized_end=4567
  _globals['_VERIFYPASSWORDLESSRESPONSE']._serialized_start=4570
  _globals['_VERIFYPASSWORDLESSRESPONSE']._serialized_end=5486
  _globals['_PASSWORDLESSSERVICE']._serialized_start=5634
  _globals['_PASSWORDLESSSERVICE']._serialized_end=9963
# @@protoc_insertion_point(module_scope)
